//
// $Id: $

package com.threerings.i18n;

import gnu.gettext.GettextResource;

import java.text.MessageFormat;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

/**
 * Provides access to translations loaded from a ResourceBundle. Plural
 * message handling is supported if the ResourceBundle was generated by
 * gettext's msgfmt.
 */
public class I18n
{
	/**
     * Mark a message for translation via xgettext. Does not translate, modify, or
     * otherwise alter the message.
     *
     * @param msgid Message string to mark for translation.
     */
    public static String mark (String msgid) {
        return msgid;
    }

    /**
     * Mark a plural message for translation via xgettext. Does not translate, modify
     * or otherwise alter the message keys.
     *
     * @param msgid Singular message string to mark for translation.
     * @param msgid_plural Plural message string to mark for translation.
     * @return An instance of PluralMessage containing the two strings.
     */
    public static PluralMessage markn (String msgid, String msgid_plural) {
    	return new PluralMessage(msgid, msgid_plural);
    }

	/**
     * Create a new GetText instance, loading translations
     * from a resource bundle matching base name and locale.
     *
     * @param baseName Passed to {@link ResourceBundle#getBundle(String, Locale)}
     * @param locale Passed to {@link ResourceBundle#getBundle(String, Locale)}
     */
    public I18n (String baseName, Locale locale)
        throws MissingResourceException
    {
        _bundle = ResourceBundle.getBundle(baseName, locale);
    }

    /**
     * Create a new GetText instance, loading translations from the provided
     * {@link ResourceBundle}.
     * @param bundle A gettext-generated {@link ResourceBundle} instance.
     */
    public I18n (ResourceBundle bundle) {
        _bundle = bundle;
    }

    /**
     * Returns the translation bundle.
     */
    public ResourceBundle getBundle () {
        return _bundle;
    }

    /**
     * Return the translation bundle's locale.
     */
    public Locale getLocale () {
    	return _bundle.getLocale();
    }

    /**
     * Set the default method of image localization, which may be overridden by I18n:img tags.
     */
    public void setImgLocalize (String imgLocalize) {
        _imgLocalize = imgLocalize;
    }

    /**
     * Return the default method of image localization
     */
    public String getImgLocalize (){
        return _imgLocalize;
    }

    /**
     * Returns the formatted translation of msgid. The translation and
     * arguments will be passed to {@link MessageFormat#format(String, Object...)}.
     *
     * As a special caveat to the {@link MessageFormat} escaping rules, any single quotes
     * are automatically escaped prior to passing the translation string to {@link MessageFormat#format},
     * as normally MessageFormat uses single quotes to escape {Integer}, which is almost certainly not
     * what the translator expects. If you do need to print {Integer}, pass it as an argument.
     *
     * If a translation for msgid is not found, msgid will be returned.
     *
     * @param msgid The translation key.
     * @param arguments Message format arguments.
     * @return The translation of msgid, or msgid if none is found.
     */
    public String gettext (String msgid, Object... arguments) {
        /* Don't attempt to translate 0-length strings */
        if (msgid.length() == 0)
            return msgid;

        /* Retrieve the translation */
        final String message = GettextResource.gettext(_bundle, msgid);

        if (arguments.length == 0) {
            /* Common case, nothing else to do */
            return message;
        } else {
            /* Argument case, escape the string and pass it off to MessageFormat */
            return MessageFormat.format(message.replace("'", "''"), arguments);
        }
    }

     /**
      * Returns the formatted plural or singular translation of msgid. The translation and
      * arguments will be passed to {@link MessageFormat#format(String, Object...)}
      *
      * As a special caveat to the {@link MessageFormat} escaping rules, any single quotes
      * are automatically escaped prior to passing the translation string to {@link MessageFormat#format},
      * as normally MessageFormat uses single quotes to escape {Integer}, which is almost certainly not
      * what the translator expects. If you do need to print {Integer}, pass it as an argument.
      *
      * If n &gt; 1, the plural form is used. Otherwise, the singular form is used.
      *
      * If a translation for msgid(_plural) is not found, msgid(_plural) will be returned.
      * @param msgid The singular translation key.
      * @param msgid_plural The plural translation key.
      * @param n The count used for singular or plural forms.
      * @param arguments Message format arguments.
      */
     public String ngettext (String msgid, String msgid_plural, long n, Object... arguments) {
         /* Don't attempt to translate 0-length strings */
         if (msgid.length() == 0)
             return msgid;

         final String message = GettextResource.ngettext(_bundle, msgid, msgid_plural, n);
         return MessageFormat.format(message.replace("'", "''"), arguments);
     }

    /** Gettext resource bundle for locale. */
    private final ResourceBundle _bundle;

    /** Default method of image localization to use */
    private String _imgLocalize = "none";
}