/*
 * Copyright 2007 Three Rings Design, Inc.
 */
package com.threerings.jsp.taglib.affiliates;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Random;
import java.util.Set;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.configuration.Configuration;

import com.samskivert.util.Logger;

import com.threerings.util.ABTest;

import com.threerings.jsp.taglib.choose.StylingTag;
import com.threerings.jsp.taglib.i18n.DefaultLocaleFilter;
import com.threerings.jsp.taglib.tracking.DefaultTrackingFilter;
import com.threerings.jsp.taglib.utils.ContextManager;

import static com.threerings.jsp.taglib.Log.log;

/**
 * Grabs visitors to a page and, based on the filter settings and that user's referral information,
 * assigns them to a test group.
 */
public class ABTestFilter implements Filter
{
    public void init(FilterConfig config)
        throws ServletException
    {
        ServletContext context = config.getServletContext();
        ContextManager contextManager = ContextManager.getContextManager(context);

        // connections to affiliate and tag repositories
        _affiliateManager = contextManager.getAffiliateManager();

        // retrieve the config settings for this filter
        Configuration abTestConfig = contextManager.getConfiguration();
        if (abTestConfig == null) {
            throw new ServletException("ABTestFilter couldn't retrieve configuration.");
        }
        if (!abTestConfig.containsKey(JSON_PATH_PROPERTY)) {
            // no configured test; init a blank list of tests and return.
            log.info("abfilter.json_path not set, not running any ab tests.");
            return;
        }
        for (ABTest test :
            ABTest.loadTests(abTestConfig.getString(JSON_PATH_PROPERTY)).values()) {
            if (test.isActive() && !test.paths.isEmpty()) {
                tests.add(test);
                if (test.affiliates != null) {
                    Set<String> affiliateIds = new HashSet<String>();
                    for (String affiliate : test.affiliates) {
                        String affiliateId;
                        if (affiliate.equals(ABTest.DIRECT_AFFILIATE)) {
                            affiliateId = affiliate;
                        } else {
                            affiliateId = _affiliateManager.parseAffiliate(affiliate);
                            if (affiliateId == null) {
                                log.warning("A/B TEST ERROR - Unknown affiliate " + affiliate
                                    + " in ABTestFilter.  Test will run on all affiliates.");
                                continue;
                            }
                        }
                        affiliateIds.add(affiliateId);
                    }
                    testAffiliateIds.put(test.id, affiliateIds);
                }
                List<String> parsedTagIds = new ArrayList<String>(test.groups.size());
                List<String> unparsedTags = new ArrayList<String>(test.groups.size());
                for (String groupId : test.groups.keySet()) {
                    parsedTagIds.add(_affiliateManager.parseTag(groupId));
                    unparsedTags.add(groupId);
                }
                testParsedGroups.put(test.id, parsedTagIds);
                testGroups.put(test.id, unparsedTags);
                log.info(Logger.format("Activating ab test", "id", test.id));
            } else {
                log.info(Logger.format("Not using inactive ab test", "id", test.id));
            }
        }
    }

    public void destroy()
    {
        // do nothing
    }

    /**
     * Verify that the visitor is eligible for a/b testing, and if so, tag them.
     */
    public void doFilter (ServletRequest servletRequest, ServletResponse servletResponse,
        FilterChain chain)
        throws IOException, ServletException
    {
        final HttpServletRequest req = (HttpServletRequest) servletRequest;
        final HttpServletResponse rsp = (HttpServletResponse) servletResponse;

        // if we are on the right page for one or more tests, execute them
        for (ABTest test : tests) {
            if (test.paths.contains(req.getServletPath())) {
                doTest(req, rsp, test);
            }
        }

        chain.doFilter(req, rsp);
    }

    /**
     * Perform an a/b test.
     */
    private void doTest (HttpServletRequest req, HttpServletResponse rsp, ABTest test)
        throws IOException
    {
        Locale locale = (Locale)req.getSession().getAttribute(DefaultLocaleFilter.LOCALE_ATTRIBUTE);
        if (test.locales != null && !test.locales.contains(locale.getLanguage())) {
            // visitor must match locale requirement to be eligible for the test
            removeTestTags(req, rsp, test);
            return;
        }

        // if visitor already has one of our test tags, let em be
        String requestTagListId = _affiliateManager.getTagListId(req);
        if (requestTagListId != null) {
            for (String groupId : testParsedGroups.get(test.id)) {
                if (_affiliateManager.tagIdListContains(requestTagListId, groupId)) {
                    // set the style again so we can use ?tag=blah for testing purposes
                    int index = testParsedGroups.get(test.id).indexOf(groupId);
                    StylingTag.setSessionStyle(req, testGroups.get(test.id).get(index), "abtest");
                    return;
                }
            }
        }

        // visitor must match referring affiliate requirements to be eligible for a/b testing
        if (!matchesAffiliate(req, test)) {
            return;
        }

        // we might not want to test on existing visitors
        if (!test.allVisitors && !DefaultTrackingFilter.isNewVisitor(req)) {
            return;
        }

        // assign a random group and set the appropriate tags
        int groupIdx = rand.nextInt(test.groups.size());
        _affiliateManager.addTagId(req, rsp, testParsedGroups.get(test.id).get(groupIdx), true);
        StylingTag.setSessionStyle(req, testGroups.get(test.id).get(groupIdx), "abtest");
    }

    /**
     * Return true if the visitor affiliate matches the one in the test, or if the visitor has no
     * affiliate and none is required, or if all affiliates are allowed.
     */
    private boolean matchesAffiliate (HttpServletRequest req, ABTest test)
    {
        // all visitors will be a/b tested regardless of affiliate
        if (test.affiliates == null) {
            return true;
        }

        // get the visitor's referring affiliate
        String requestAffiliateId = _affiliateManager.getAffiliateId(req);

        // If they don't have an affiliate, we're good if we're allowing direct
        if (requestAffiliateId == null) {
            return testAffiliateIds.get(test.id).contains(ABTest.DIRECT_AFFILIATE);
        }

        // if we only allow particular affiliate(s), the request affiliate must be one of them
        return testAffiliateIds.get(test.id).contains(requestAffiliateId);
    }

    /**
     * The visitor is no longer a member of the test, remove them from any test groups.
     */
    private void removeTestTags (HttpServletRequest req, HttpServletResponse rsp, ABTest test)
    {
        final String requestTagListId = _affiliateManager.getTagListId(req);
        if (requestTagListId == null) {
            return;
        }
        for (String id: testParsedGroups.get(test.id)) {
            if (_affiliateManager.tagIdListContains(requestTagListId, id)) {
                String newTagListId = _affiliateManager.removeTagId(requestTagListId, id);
                _affiliateManager.setTagIdList(req, rsp, newTagListId, true);
                StylingTag.setSessionStyle(req, null, "abtest");
            }
        }
    }

    protected static final String JSON_PATH_PROPERTY = "abfilter_json_path";

    /** Provides a connection to the affiliate repos and utils for tracking affiliate referrals. */
    private AffiliateManager _affiliateManager;

    /** List of all the tests currently running */
    private List<ABTest> tests = new ArrayList<ABTest>();

    /** Map from test id to a list of its parsed tag ids. */
    private Map<String, List<String>> testParsedGroups = new HashMap<String, List<String>>();

    /** Map from test id to a list of its group ids. */
    private Map<String, List<String>> testGroups = new HashMap<String, List<String>>();

    /** Map from test id to its affiliates. */
    private Map<String, Set<String>> testAffiliateIds = new HashMap<String, Set<String>>();

    /** For selecting a random group */
    private final Random rand = new Random();
}
