package com.threerings.jsp.taglib.affiliates;

import java.io.IOException;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.tagext.JspFragment;
import javax.servlet.jsp.tagext.SimpleTagSupport;

/**
 * Tag representing an if statement based upon affiliate, to be included within a
 * {@link AffiliateChooseTag}. If the current affiliate matches the affiliate attribute, the body
 * of this tag will be evaluated. If two or more {@link AffiliateCaseTag}s in the same choose tag
 * match, the first will be evaluated. If the affiliate attribute is set to "default", it will
 * only be evaluated if no other case tag matches.
 */
public class AffiliateCaseTag extends SimpleTagSupport
{
    /**
     * Invoke the body of this tag.  This will be called by the parent upon selecting this
     * tag as a match.
     */
    public void doBody()
        throws JspException, IOException
    {
        final JspFragment body = getJspBody();
        body.invoke(null);
    }

    /**
     * Assess how well the type attribute matches the visitor type in the parent tag, then
     * register this value with the parent {@link AffiliateChooseTag}.
     */
    @Override
    public void doTag()
        throws JspException, IOException
    {
        // this must appear within the correct parent tag
        parentTag = (AffiliateChooseTag)findAncestorWithClass(this, AffiliateChooseTag.class);
        if (parentTag == null) {
            throw new JspTagException("Affiliate case tags must be in an affiliate choose tag.");
        }

        // determine weak or strong match value
        int totalMatchValue = 0;
        for (String affiliate : _affiliates) {
            if (affiliate.equals("none")) {
                totalMatchValue += (parentTag.affiliateMatches(null)) ? 3 : 0;
            } else if (affiliate.equals("default")) {
                totalMatchValue += 1;
            } else if (affiliate.equals("*")) {
                totalMatchValue += (parentTag.affiliateMatches(null)) ? 0 : 2;
            } else {
                totalMatchValue += (parentTag.affiliateMatches(affiliate)) ? 3 : 0;
            }
        }

        // if this matches the current affiliate, notify parent
        if (totalMatchValue > 0) {
            parentTag.setMatch(totalMatchValue, this);
        }
    }

    /**
     * Set the affiliate to match against (eg "miniclip"), or "none" for no affiliate, or "*" to
     * match some (not direct) affiliate, "default" to weakly match anything. May be a
     * comma-delimeted list of affiliates.
     */
    public void setAffiliate (String affiliates)
    {
        _affiliates = affiliates.split(",");
        for (int i = 0; i < _affiliates.length; i++) {
            _affiliates[i] = _affiliates[i].trim();
        }
    }

    /**
     * affiliate string (eg "miniclip" or "miniclip,miniclip_de") to match against the current
     * affiliate
     */
    private String[] _affiliates;

    /** The parent tag */
    private AffiliateChooseTag parentTag;
}