package com.threerings.jsp.taglib.affiliates;

import java.io.IOException;
import java.io.StringWriter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.JspFragment;
import javax.servlet.jsp.tagext.SimpleTagSupport;

import com.threerings.jsp.taglib.affiliates.AffiliateManager;
import com.threerings.jsp.taglib.utils.ContextManager;

/**
 * Contains one or more {@link AffiliateCaseTag}s. Only the case tag that matches the correct
 * affiliate string will be displayed. If an affiliate string is unknown, this tag will also look
 * for an override string in the request params (eg ?sitename=miniclip).
 */
public class AffiliateChooseTag extends SimpleTagSupport
{
    @Override
    public void doTag()
        throws JspException, IOException
    {
        // find the current affiliate
        final PageContext pageContext = (PageContext)getJspContext();
        final HttpServletRequest req = (HttpServletRequest)pageContext.getRequest();
        final ContextManager contextManager = ContextManager.getContextManager(pageContext);
        _affiliateManager = contextManager.getAffiliateManager();

        // Check for an override affiliate string/name under different affiliate parameters
        // if one is present, use it instead of the actual current affiliate, for testing in
        // environments that don't have all affiliates in the database
        for (String paramName : _affiliateManager.getAffiliateParamNames()) {
            _affiliateOverrideString = req.getParameter(paramName);
            if (_affiliateOverrideString != null) {
                _affiliateOverrideString = _affiliateOverrideString.trim();
                break;
            }
        }

        // there is no override, so fetch the actual affiliate.
        if (_affiliateOverrideString == null) {
            _affiliateId = _affiliateManager.getAffiliateId(req);
        }

        // invoke the body to trigger child tag matching (children will call affiliateMatches now)
        final JspFragment body = getJspBody();
        final StringWriter writer = new StringWriter();
        body.invoke(writer);

        // invoke the body of the best matched child tag
        if (_matchedTag != null) {
            _matchedTag.doBody();
        }
    }

    /**
     * Called by {@link AffiliateCaseTag} when a child tag matches. Only the child tag with the
     * highest matchValue will be kept and evaluated later, or the first in sequence if more than
     * one match equally.
     * @param matchValue Integer ranking the closeness of the match.
     * @param matchedTag Instance of the child tag that made the match.
     */
    protected void setMatch (int matchValue, AffiliateCaseTag matchedTag)
    {
        if (matchValue < 1) {
            return;
        }
        if (_matchedTag == null || matchValue > _matchValue) {
            _matchValue = matchValue;
            _matchedTag = matchedTag;
        }
    }

    /**
     * @param affiliateString String that can be turned into an affiliateId (eg "miniclip")
     * @return true if affiliateString matches the current (or overriding) affiliate, or if
     * affiliateString is null and there is no affiliate.
     */
    public boolean affiliateMatches (String affiliateString)
    {
        // compare the affiliate name to our override string from the request param
        if (_affiliateOverrideString != null) {
            return (_affiliateOverrideString.equals(affiliateString));

        // parse the affiliate name and compare affiliate ids
        } else if (_affiliateId != null) {
            String affiliateId = _affiliateManager.parseAffiliate(affiliateString);
            return (affiliateId != null && affiliateId.equals(_affiliateId));

        // no overriding or current affiliate, match only if affiliateString is null too
        } else {
            return (affiliateString == null);
        }
    }

    /** Value of the current highest-ranking match */
    private int _matchValue;

    /** Child tag that made the current highest-ranking match */
    private AffiliateCaseTag _matchedTag;

    /** Current affiliate identifier */
    private String _affiliateId;

    /** If there is an affiliateString in the request params use that instead of the
     * current affiliate, for testing. (eg ?sitename=miniclip) */
    private String _affiliateOverrideString;

    /** Can convert between affiliateString and affiliateId */
    private AffiliateManager _affiliateManager;
}