/*
 * Copyright 2007 Three Rings Design, Inc.
 * Copyright 1999,2004 The Apache Software Foundation.
 */

package com.threerings.jsp.taglib.affiliates;

import java.io.IOException;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.threerings.jsp.taglib.utils.ContextManager;

/**
 * Servlet filter to find and record the referral information for the current visitor.
 * Uses com.threerings.user.AffiliateInfo to parse request parameters and set cookies,
 * and runs alongside legacy affiliate tagging in Velocity affiliate pages.
 */
public class AffiliateFilter implements Filter {

    /**
     * Filter initialization.  Retrieve configuration defaults.
     */
    public void init(FilterConfig config)
        throws ServletException
    {
        final ServletContext context = config.getServletContext();
        final ContextManager contextManager = ContextManager.getContextManager(context);

        // affiliate manager and utils for accessing affiliate data
        _affiliateManager = contextManager.getAffiliateManager();

        // should we strip the hostname off our domain? Default to true.
        String param = config.getInitParameter("strip-cookie-domain");
        _cookieStripDomain = param == null || param.equalsIgnoreCase("true");
    }

    /**
     * Filter destruction.
     */
    public void destroy()
    {
        // do nothing
    }

    /**
     * If an affiliate request parameter (eg ?affiliate=miniclip) is present, match it up to
     * an affiliate in our repositories, and store it as a cookie and in the session.
     * Also check for a second affiliate tag parameter used to track the effectiveness of
     * different links on the same referrer's site.  If there are no request parameters, and
     * the affiliate isn't stored in the session, check the visitor's cookies for an old
     * affiliate id.
     */
    public void doFilter (ServletRequest servletRequest, ServletResponse servletResponse, FilterChain chain)
        throws IOException, ServletException
    {
        final HttpServletRequest req = (HttpServletRequest) servletRequest;
        final HttpServletResponse rsp = (HttpServletResponse) servletResponse;

        // get the affiliate from a request parameter
        final String newAffiliateId = getAffiliateId(req);
        if (newAffiliateId != null) {
            if (newAffiliateId.equals("none")) {
                _affiliateManager.setAffiliateId(req, rsp, null, _cookieStripDomain);
            }
            else {
                _affiliateManager.setAffiliateId(req, rsp, newAffiliateId, _cookieStripDomain);
            }
        }
        // get the affiliate from a cookie if it's missing from the sesssion
        else {
            _affiliateManager.updateSessionAffiliateFromCookies(req);
        }

        // Also save the tag if present in the request parameters
        final String newAffiliateTag = getAffiliateTag(req);
        if (newAffiliateTag != null) {
            _affiliateManager.addTagId(req, rsp, newAffiliateTag, _cookieStripDomain);
        }
        // or get it out of the cookie if present
        else {
            _affiliateManager.updateSessionTagIdListFromCookies(req);
        }

        // pass the request on to the next filter
        chain.doFilter(req, rsp);
    }

    /**
     * Retrieve the affiliate request parameter, or null if not present.  First checks the
     * new affiliate parameter, then the legacy affiliate/referral "from" parameter.  Param is
     * then parsed via the AffiliateManager to give us the proper affiliate identifier.
     * @return Identifier of an affiliate, or null if there is none, or "none" if explicitely
     *         set to "none".
     */
    private String getAffiliateId (HttpServletRequest req)
    {
        // Check for the affiliate name under different affiliate parameters
        String affiliate = null;
        for (String paramName : _affiliateManager.getAffiliateParamNames()) {
            affiliate = req.getParameter(paramName);
            if (affiliate != null) {
                break;
            }
        }

        if (affiliate != null) {
            if (affiliate.equals("none")) {
                return "none";
            }
            return _affiliateManager.parseAffiliate(affiliate);
        }

        // It's possible the affiliate id will be passed in as a param
        String affiliateId = req.getParameter(_affiliateManager.getAffiliateIdParamName());
        if (affiliateId != null) {
            return affiliateId;
        }
        return null;
    }

    /**
     * Retrieve the affiliate tag request parameter and parse it to get the proper tag
     * identifier, or null if not present.
     * @return Arbitrary tag string used to differentiate multiple links from a single affiliate.
     */
    private String getAffiliateTag (HttpServletRequest req)
    {
        String tag = req.getParameter(_affiliateManager.getAffiliateTagParamName());
        if (tag != null) {
            return _affiliateManager.parseTag(tag);
        }
        return null;
    }

    /** Provides a connection to the affiliate repo and utils for tracking affiliate referrals. */
    private AffiliateManager _affiliateManager;

    /** Should we remove the domain (eg "www.") from new cookies? */
    private boolean _cookieStripDomain;
}