/*
 * Copyright 2007 Three Rings Design, Inc.
 */

package com.threerings.jsp.taglib.affiliates;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.threerings.jsp.taglib.utils.CookieUtils;

/**
 * Utilities for dealing with affiliate tagging.
 */
public abstract class AffiliateManager
{
    /**
     * Validate an affiliate request parameter, and return the affiliate identifier (affiliateId),
     * or null if affiliate is invalid.
     */
    public abstract String parseAffiliate (String affiliate);

    /**
     * Validate an affiliate tag parameter, and return an identifier (tagId) or null if tag is
     * invalid.
     */
    public abstract String parseTag (String tag);

    /**
     * Retrieve the list of affiliate parameter names (for affiliate strings)
     */
    public abstract List<String> getAffiliateParamNames ();

    public abstract String getAffiliateIdParamName ();

    public abstract String getAffiliateTagParamName ();

    public abstract String getAffiliateIdAttributeName ();

    public abstract String getTagIdListAttributeName ();

    public abstract String getAffiliateIdCookieName ();

    public abstract String getTagIdListCookieName ();

    /**
     * Retrieve the default style for the affiliate in the request.
     */
    public abstract String getAffiliateStyle (HttpServletRequest req);

    /**
     * Save the affiliate id to the session and cookies. If affiliateId is null, instead clear the
     * session and cookie
     */
    public void setAffiliateId (HttpServletRequest req, HttpServletResponse rsp,
        String affiliateId, boolean cookieStripDomain)
    {
        setSessionAttribute(req, getAffiliateIdAttributeName(), affiliateId);
        setCookie(req, rsp, getAffiliateIdCookieName(), affiliateId, cookieStripDomain);
    }

    /**
     * Look in session, then cookies to get visitor's affiliate, or return null if there isn't one.
     */
    public String getAffiliateId (HttpServletRequest req)
    {
        String tag = getSessionAttribute(req, getAffiliateIdAttributeName());
        if (tag == null) {
            tag = CookieUtils.getCookie(req, getAffiliateIdCookieName());
        }
        return tag;
    }

    /**
     * If affiliate is present in cookies but not the session, copy the cookie value into the session.
     */
    public void updateSessionAffiliateFromCookies (HttpServletRequest req)
    {
        String cookieAffiliateId = CookieUtils.getCookie(req, getAffiliateIdCookieName());
        if (getSessionAttribute(req, getAffiliateIdAttributeName()) == null
            && cookieAffiliateId != null) {
            setSessionAttribute(req, getAffiliateIdAttributeName(), cookieAffiliateId);
        }
    }

    /**
     * Returns true if a list of tag ids contains a particular tag id.
     * @param tagId Single tag identifier to search for, as returned by a call to parseTag
     */
    public boolean tagIdListContains (String tagIdList, String tagId) {

        return tagIdList != null && tagIdList.equals(tagId);
    }

    /**
     * We don't concatenate tags, we replace them.  Return null if tagListId equaled tagId.
     *
     * @param tagId Single tag identifier to remove
     * @return The new tagListId without tagId in it.
     */
    public String removeTagId (String tagIdList, String tagId)
    {
        if (tagIdList.equals(tagId)) {
            return null;
        }
        return tagIdList;
    }

    /**
     * Save a tag id to the session and cookies, or append it to any existing tag there. If tagId
     * is null, do nothing.
     *
     * @param tagId Id of a single tag string, as returned by parseTag
     */
    public void addTagId (HttpServletRequest req, HttpServletResponse rsp, String tagId,
        boolean cookieStripDomain)
    {
        if (tagId == null || tagIdListContains(getTagListId(req), tagId)) {
            return;
        }
        setTagIdList(req, rsp, tagId, cookieStripDomain);
    }

    /**
     * Save the tag list to the session and cookies, overwriting anything there. If the tagIdList
     * is null, instead remove the cookie.
     */
    public void setTagIdList (HttpServletRequest req, HttpServletResponse rsp, String tagIdList,
        boolean cookieStripDomain)
    {
        setSessionAttribute(req, getTagIdListAttributeName(), tagIdList);
        setCookie(req, rsp, getTagIdListCookieName(), tagIdList, cookieStripDomain);
    }

    /**
     * Look in session, then cookies to get visitor's tags, or null if there isn't one.
     */
    public String getTagListId (HttpServletRequest req)
    {
        String tag = getSessionTagIdList(req);
        if (tag == null) {
            return CookieUtils.getCookie(req, getTagIdListCookieName());
        }
        return tag;
    }

    /**
     * If tag is present in cookies but not the session, copy the cookie value into the session.
     */
    public void updateSessionTagIdListFromCookies (HttpServletRequest req)
    {
        String cookieTagId = CookieUtils.getCookie(req, getTagIdListCookieName());
        if (getSessionTagIdList(req) == null && cookieTagId != null) {
            setSessionAttribute(req, getTagIdListAttributeName(), cookieTagId);
        }
    }

    protected String getSessionTagIdList (HttpServletRequest req)
    {
        return getSessionAttribute(req, getTagIdListAttributeName());
    }

    protected String getSessionAttribute (HttpServletRequest req, String attributeName)
    {
        return (String)req.getSession().getAttribute(attributeName);
    }

    protected void setSessionAttribute (HttpServletRequest req, String attributeName, String value)
    {
        req.getSession().setAttribute(attributeName, value);
    }

    /**
     * Sets the given cookie name to the given value, or deletes it if the value is null.
     */
    protected void setCookie (HttpServletRequest req, HttpServletResponse rsp, String cookieName,
        String value, boolean cookieStripDomain)
    {
        if (value != null) {
            CookieUtils.setCookie(req, rsp, cookieName, value, 2 * 365, cookieStripDomain);
        } else {
            CookieUtils.deleteCookie(req, rsp, cookieName, cookieStripDomain);
        }
    }
}