/*
 * Copyright 2007 Three Rings Design, Inc.
 */
package com.threerings.jsp.taglib.affiliates;

import java.io.Writer;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.SimpleTagSupport;

import org.apache.commons.configuration.Configuration;
import org.apache.commons.lang.StringEscapeUtils;

import com.threerings.jsp.taglib.utils.ContextManager;

/**
 * Print a 1x1 pixel tracking image with the given src attribute for the appropriate affiliate.
 * Fetches affiliate and image locations from a ConfigurationManager instance.
 */
public class AffiliateTrackerTag extends SimpleTagSupport
{
    @Override
    @SuppressWarnings("unchecked")
    public void doTag ()
        throws JspException
    {
        final PageContext pageContext = (PageContext) getJspContext();
        final HttpServletRequest req = (HttpServletRequest) pageContext.getRequest();
        final ContextManager contextManager = ContextManager.getContextManager(pageContext);

        // connections to repos and cookie get/set methods
        final AffiliateManager affiliateManager = contextManager.getAffiliateManager();
        final String requestAffiliateId = affiliateManager.getAffiliateId(req);

        // if there is no affiliate, there are no trackers to display so leave
        if (requestAffiliateId == null) {
            return;
        }

        // retrieve the config settings for the tracker
        Configuration config = getConfig(contextManager);
        if (config == null || !config.containsKey(_name)) {
            // don't log an error if there are no settings for this tracker
            return;
        }

        // for each affiliate that we have tracking sprite(s) for, if it matches the affiliate
        // in the request, display all tracking sprites for the affiliate
        String output = "";
        final List<String> affiliates = config.getList(_name);
        for (String affiliate : affiliates) {
            final String affiliateId = affiliateManager.parseAffiliate(affiliate);
            if (affiliateId != null && affiliateId.equals(requestAffiliateId)) {
                final List<String> urls = config.getList(_name + "." + affiliate + ".url");
                for (String url :urls) {
                    String escapedURL = StringEscapeUtils.escapeHtml(url);
                    output += "<img src=\"" + escapedURL + "\" width=\"1\" height=\"1\" />\n";
                }
            }
        }
        // there was nothing to output, so leave
        if (output.length() == 0) {
            return;
        }

        // If we make it to here, print the tracking sprite
        final Writer out = getJspContext().getOut();
        try {
            out.write(output);
        } catch (final Exception e) {
            throw new JspTagException("AffiliateTrackerTag couldn't produce output", e);
        }
    }

    /**
     * Fetch the configuration properties for the affiliate tracker tag.
     * Expecting properties in the format:
     *     tracker.[tracker name].[affiliate string].url = [image url with query string]
     */
    private Configuration getConfig (ContextManager contextManager)
        throws JspTagException
    {
        if (_name == null) {
            return null;
        }
        final Configuration config = contextManager.getConfiguration();
        if (config == null || !config.containsKey(CONFIG_BASE)) {
            return null;
        }
        return config.subset(CONFIG_BASE);
    }

    /**
     * Set the name of this tracker, which will be used to look up config settings
     */
    public void setName (String name) {
        _name = name;
    }
    /** Tracker name under which the config settings will be stored */
    private String _name;

    /** Root location of the configuration properties for the affiliate tracker tag */
    private static final String CONFIG_BASE = "tracker";
}