package com.threerings.jsp.taglib.agent;

import java.io.IOException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.JspFragment;
import javax.servlet.jsp.tagext.SimpleTagSupport;

/**
 * Tag representing an if statement based upon a particular web browser.
 * If the user-agent request header matches the browser type and version,
 * the body of this tag will be evaluated.  If included within a BrowserTag,
 * body will additionally only be evaluated if a match was not made with
 * a BrowserCaseTag earlier in the sequence.
 */
public abstract class BrowserCaseTag extends SimpleTagSupport
{
    /**
     * Default tag always matches any user agent, but may only be used
     * within a BrowserTag.
     */
    public static class DefaultBrowserTag extends BrowserCaseTag
    {
        @Override
        protected int matchAgent (String userAgent) {
            return 1;
        }
    }

    /**
     * MS Internet Explorer browser case tag.  Will also match AOL browsers
     * (AOL 8/9 = MSIE 6, AOL 7/6 = MSIE 5.5)
     */
    public static class MSIETag extends BrowserCaseTag
    {
        @Override
        protected int matchAgent (String userAgent)
            throws JspException
        {
            if (!userAgent.contains("msie")) {
                return 0;
            }

            if (version == null) {
                return 2;
            }
            else if (version.equals("7")) {
                if (userAgent.contains("msie 7")) {
                    return 3;
                }
                return 0;
            }
            else if (version.equals("6")) {
                if (userAgent.contains("msie 6")) {
                    return 3;
                }
                return 0;
            }
            else if (version.equals("5.5")) {
                if (userAgent.contains("msie 5.5")) {
                    return 3;
                }
                return 0;
            }
            throw new JspException("Unknown browser version:" + version);
        }
    }

    /**
     * Firefox browser case tag.  May not identify Firefox running on Linux,
     * as it identifies itself as Mozilla instead of Firefox.
     */
    public static class FirefoxTag extends BrowserCaseTag
    {
        @Override
        protected int matchAgent (String userAgent)
            throws JspException
        {
            if (!userAgent.contains("firefox")) {
                return 0;
            }

            if (version == null) {
                return 2;
            }
            else if (version.equals("2")) {
                if (userAgent.contains("firefox/2")) {
                    return 3;
                }
                return 0;
            }
            else if (version.equals("1.5")) {
                if (userAgent.contains("firefox/1.5")) {
                    return 3;
                }
                return 0;
            }
            throw new JspException("Unknown browser version:" + version);
        }
    }

    /**
     * Safari browser case tag.  Versions not currently supported; they do not
     * correspond nicely, eg 3.0 == Safari/521, 2.0 == Safari/401.
     */
    public static class SafariTag extends BrowserCaseTag
    {
        @Override
        protected int matchAgent (String userAgent)
            throws JspException
        {
            if (!userAgent.contains("safari")) {
                return 0;
            }
            if (version == null) {
                return 2;
            }
            throw new JspException("Unknown browser version:" + version);
        }
    }

    /**
     * Netscape browser case tag.  Versions prior to 7 will not match, as they
     * identified themselves as Mozilla instead of Netscape.
     */
    public static class NetscapeTag extends BrowserCaseTag
    {
        @Override
        protected int matchAgent (String userAgent)
            throws JspException
        {
            if (!userAgent.contains("netscape")) {
                return 0;
            }

            if (version == null) {
                return 2;
            }
            else if (version.equals("8")) {
                if (userAgent.contains("netscape/8")) {
                    return 3;
                }
                return 0;
            }
            else if (version.equals("7")) {
                if (userAgent.contains("netscape/7")) {
                    return 3;
                }
                return 0;
            }
            throw new JspException("Unknown browser version:" + version);
        }
    }

    /**
     * Opera browser case tag.
     */
    public static class OperaTag extends BrowserCaseTag
    {
        @Override
        protected int matchAgent (String userAgent)
            throws JspException
        {
            if (!userAgent.contains("opera")) {
                return 0;
            }

            if (version == null) {
                return 2;
            }
            else if (version.equals("9")) {
                if (userAgent.contains("opera/9") || userAgent.contains("opera 9")) {
                    return 3;
                }
                return 0;
            }
            else if (version.equals("8")) {
                if (userAgent.contains("opera/8") || userAgent.contains("opera 8")) {
                    return 3;
                }
                return 0;
            }
            throw new JspException("Unknown browser version:" + version);
        }
    }

    /**
     * Browser case tag to match common search engine bots.  Does not
     * accept a version attribute.
     */
    public static class RobotTag extends BrowserCaseTag
    {
        @Override
        protected int matchAgent (String userAgent)
            throws JspException
        {
            String[] botlist = {"googlebot", "msnbot", "yahoo! slurp"};
            for (int i = 0; i < botlist.length; i++) {
                if (userAgent.contains(botlist[i])) {
                    return 2;
                }
            }
            return 0;
        }
    }

    /**
     * Invoke the body of this tag.  If within a {@link BrowserTag}, this
     * may be called by the parent upon selecting this tag as a match.
     */
    public void doBody()
        throws JspException, IOException
    {
        final JspFragment body = getJspBody();
        body.invoke(null);
    }

    @Override
    public void doTag()
        throws JspException, IOException
    {
        // this must appear within a browser tag
        if (getParent() != null && !(getParent() instanceof BrowserTag)) {
            throw new JspTagException("Default browser only valid within a browser tag.");
        }
        parentBrowserTag = (BrowserTag) getParent();

        // Get the visitor's User Agent
        PageContext pageContext = (PageContext) getJspContext();
        HttpServletRequest request = (HttpServletRequest) pageContext.getRequest();
        String userAgent = request.getHeader("User-Agent");
        userAgent = (userAgent == null) ? "" : userAgent.toLowerCase();

        // if visitor has the right browser, notify parent
        int matchValue = matchAgent(userAgent);
        if (matchValue > 0) {
            // notify the parent we've matched
            parentBrowserTag.setMatch(matchValue, this);
        }
    }

    /**
     * Set the browser version attribute (eg "7" or "5.5")
     */
    public void setVersion (String version)
        throws JspException
    {
        this.version = version;
    }

    /**
     * Returns a value if a match was found between this browser
     * and the user agent, zero otherwise.  If the version attribute
     * is set, the value returned will be higher on a complete match,
     * zero for a partial or no match.
     * @param userAgent Lower case user-agent string
     * @return 0 if no match is found, 1 for a default tag match,
     *     2 for a match on one variable (browser name),
     *     3 for a match on two variables (browser and version), etc
     */
    protected abstract int matchAgent (String userAgent)
        throws JspException;

    /** attribute determining the browser version to match against */
    protected String version;

    /** The parent browser tag, or null if parent is not a browser tag */
    protected BrowserTag parentBrowserTag;
}
