package com.threerings.jsp.taglib.agent;

import java.io.IOException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.JspFragment;
import javax.servlet.jsp.tagext.SimpleTagSupport;

/**
 * Tag representing an if statement based upon a particular operating system.
 * If the user-agent request header matches the platform type and version,
 * the body of this tag will be evaluated.  If included within a PlatformTag,
 * body will additionally only be evaluated if a match was not made with
 * a PlatformCaseTag earlier in the sequence.
 *
 * Robots/spiders are classified as a {@link BrowserCaseTag}.
 */
public abstract class PlatformCaseTag extends SimpleTagSupport
{
    /**
     * Default tag always matches any user agent, but may only be used
     * within a {@link PlatformTag}.
     */
    public static class DefaultPlatformTag extends PlatformCaseTag
    {
        @Override
        protected int matchAgent (String userAgent) {
            return 1;
        }
    }

    /**
     * Microsoft Windows platform tag.  Covers Vista, XP, 2003, 2000, 98
     * and 95 releases.
     */
    public static class WindowsTag extends PlatformCaseTag
    {
        @Override
        protected int matchAgent (String userAgent)
            throws JspException
        {
            if (!userAgent.contains("windows") && !userAgent.contains("win9")) {
                return 0;
            }

            if (release == null) {
                return 2;
            }
            else if (release.equals("vista")) {
                if (userAgent.contains("windows nt 6.0")) {
                    return 3;
                }
                return 0;
            }
            else if (release.equals("xp")) {
                if (userAgent.contains("windows nt 5.1") || userAgent.contains("windows xp")) {
                    if (version == null) {
                        return 3;
                    }
                    // service pack 2 == sv1 (security version 1)
                    // only MSIE puts this token in here
                    if (version.equals("sp2")) {
                        if (userAgent.contains("sv1")) {
                            return 4;
                        }
                        return 0;
                    }
                    throw new JspException("Unknown Windows XP version:" + version);
                }
                return 0;
            }
            else if (release.equals("2003")) {
                if (userAgent.contains("windows nt 5.2")) {
                    return 3;
                }
                return 0;
            }
            else if (release.equals("2000")) {
                if (userAgent.contains("windows nt 5.0")) {
                    return 3;
                }
                return 0;
            }
            else if (release.equals("98")) {
                if (userAgent.contains("win98")) {
                    return 3;
                }
                return 0;
            }
            else if (release.equals("95")) {
                if (userAgent.contains("win95")) {
                    return 3;
                }
                return 0;
            }
            throw new JspException("Unknown platform release:" + release);
        }
    }

    /**
     * Macintosh platform tag.  Supports "osx" and "ppc" releases.
     */
    public static class MacTag extends PlatformCaseTag
    {
        @Override
        protected int matchAgent (String userAgent)
            throws JspException
        {
            if (!userAgent.contains("macintosh") && !userAgent.contains("mac os")) {
                return 0;
            }

            if (release == null) {
                return 2;
            }
            else if (release.equals("osx")) {
                if (userAgent.contains("mac os x")) {
                    return 3;
                }
                return 0;
            }
            else if (release.equals("ppc")) {
                if (userAgent.contains("mac_ppc") || userAgent.contains("mac_powerpc")) {
                    return 3;
                }
                return 0;
            }
            throw new JspException("Unknown platform release:" + release);
        }
    }

    /**
     * Linux platform case tag.  Most newer flavours specify "linux" in
     * the user-agent.  Does not accept a release or version attribute.
     */
    public static class LinuxTag extends PlatformCaseTag
    {
        @Override
        protected int matchAgent (String userAgent)
            throws JspException
        {
            String[] flavorList = {"linux", "freebsd", "irix", "sunos", "beos"};
            for (int i = 0; i < flavorList.length; i++) {
                if (userAgent.contains(flavorList[i])) {
                    return 2;
                }
            }
            return 0;
        }
    }


    /**
     * Invoke the body of this tag.  If within a {@link PlatformTag}, this
     * may be called by the parent upon selecting this tag as a match.
     */
    public void doBody()
        throws JspException, IOException
    {
        final JspFragment body = getJspBody();
        body.invoke(null);
    }

    @Override
    public void doTag()
        throws JspException, IOException
    {
        // this must appear within a platform tag
        if (getParent() != null && !(getParent() instanceof PlatformTag)) {
            throw new JspTagException("Default platform only valid within a platform tag.");
        }
        parentPlatformTag = (PlatformTag) getParent();

        // Get the visitor's User Agent
        PageContext pageContext = (PageContext) getJspContext();
        HttpServletRequest request = (HttpServletRequest) pageContext.getRequest();
        String userAgent = request.getHeader("User-Agent");
        userAgent = (userAgent == null) ? "" : userAgent.toLowerCase();

        // if visitor has the right platform, notify parent
        int matchValue = matchAgent(userAgent);
        if (matchValue > 0) {
            // notify the parent we've matched
            parentPlatformTag.setMatch(matchValue, this);
        }
    }

    /**
     * Set the platform release attribute, eg "xp" or "vista".
     */
    public void setRelease (String release)
        throws JspException
    {
        this.release = release;
    }


    /**
     * Set the platform sub-release version attribute, eg "sp2".
     */
    public void setVersion (String version)
        throws JspException
    {
        this.version = version;
    }

    /**
     * Returns a value if a match was found between this platform
     * and the user agent, zero otherwise.  If the version attribute
     * is set, the value returned will be higher on a complete match,
     * zero for a partial or no match.
     * @param userAgent Lower case user-agent string
     * @return 0 if no match is found, 1 for a default tag match,
     *     2 for a match on one variable (platform name),
     *     3 for a match on two variables (platform and release),
     *     4 for a match on platform, release and version.
     */
    protected abstract int matchAgent (String userAgent)
        throws JspException;

    /** attribute determining the release (eg "vista") to match against */
    protected String release;

    /** attribute determining the sub-release version (eg "sp2") to match against */
    protected String version;

    /** The parent platform tag, or null if parent is not a platform tag */
    protected PlatformTag parentPlatformTag;
}
