package com.threerings.jsp.taglib.choose;

import java.io.IOException;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.tagext.JspFragment;
import javax.servlet.jsp.tagext.SimpleTagSupport;

/**
 * Tag representing an if statement based upon a particular style, to be included within a
 * {@link StylingTag}. If the current style in the StylingTag matches the style attribute, the
 * body of this tag will be evaluated. If two or more {@link StyleTag}s in the same styling tag
 * match, the first will be evaluated. If the style attribute is set to "default", it will only be
 * evaluated if no other style tag matches. If the style attribute is set to "*", it will be
 * evaluated if a style is active and there isn't an exact match for the active style name.
 */
public class StyleTag extends SimpleTagSupport
{
    /**
     * Invoke the body of this tag.  This will be called by the parent upon selecting this
     * tag as a match.
     */
    public void doBody()
        throws JspException, IOException
    {
        final JspFragment body = getJspBody();
        if (body != null) {
            body.invoke(null);
        }
    }

    /**
     * Assess how well the style attribute matches the style in the parent tag, then register
     * this value with the parent {@link StylingTag}.
     */
    @Override
    public void doTag()
        throws JspException, IOException
    {
        // this must appear within the correct parent tag
        parentTag = (StylingTag) findAncestorWithClass(this, StylingTag.class);
        if (parentTag == null) {
            throw new JspTagException("style:style tags only valid within a style:styling tag.");
        }

        // if this matches the current style (or the default), notify parent
        int matchValue = matchStyle(parentTag.getStyle());
        if (matchValue > 0) {
            // notify the parent we've matched
            parentTag.setMatch(matchValue, this);
        }
    }

    /**
     * Set the style name to match against, or "default" to weakly match any style
     */
    public void setName (String name)
    {
        _name = name;
    }

    /**
     * Returns a value if a match was found between the style attribute and the current style,
     * or zero for no match.  The higher the value, the better the match.
     *
     * @param currentStyle Current style to match style attribute against
     * @return 0 if no match is found, 1 for a default tag match, 2 for a matching style.
     */
    protected int matchStyle (String currentStyle)
        throws JspException
    {
        if (_name == null) {
            return 0;
        } else if (_name.equalsIgnoreCase(currentStyle)) {
            // if the style attribute matches, return a strong match value, including if this is
            // default matching against default
            return 3;
        } else if (_name.equalsIgnoreCase("default")) {
            // if the style attribute is default, return a weak match value
            return 1;
        } else if (currentStyle == null || currentStyle.equals("default")) {
            return 0;
        } else if(_name.equals("*")) {
            // if we're matching anything with a style, return a middlin' match value
            return 2;
        } else {
            return 0;
        }
    }


    /** name attribute to match against the current style */
    private String _name;

    /** The parent tag */
    protected StylingTag parentTag;
}
