package com.threerings.jsp.taglib.choose;

import java.io.IOException;
import java.io.StringWriter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.JspFragment;
import javax.servlet.jsp.tagext.SimpleTagSupport;

import com.threerings.jsp.taglib.affiliates.AffiliateManager;
import com.threerings.jsp.taglib.utils.ContextManager;

/**
 * Contains one or more {@link StyleTag}s.  Only the style tag that best matches the current
 * style settings - as calculated from a request variable - will be evaluated.
 */
public class StylingTag extends SimpleTagSupport
{
    /**
     * Save the style in the appropriate session attribute for the given type, for use
     * by other tags that want to set a style.
     */
    public static void setSessionStyle (HttpServletRequest req, String style, String type) {
        if (type != null && type.equals("abtest")) {
            req.getSession().setAttribute(STYLE_ATTRIBUTE_ABTEST, style);
        }
        else {
            req.getSession().setAttribute(STYLE_ATTRIBUTE_AFFILIATE, style);
        }
    }

    /**
     * Determine the current style for the request, then trigger child StyleTags to match against
     * it.  Invoke the body of the best-matched child tag.
     */
    @Override
    public void doTag()
        throws JspException, IOException
    {
        // calculate the current style
        determineStyle();
        // invoke the body to trigger child tag matching
        final JspFragment body = getJspBody();
        final StringWriter writer = new StringWriter();
        body.invoke(writer);

        // invoke the body of the best matched child tag
        if (_matchedTag != null) {
            _matchedTag.doBody();
        }
    }

    /**
     * Called by {@link StyleTag} when a child tag matches the current style.  Only the child
     * tag with the highest matchValue will be kept and evaluated later, or the first in sequence
     * if more than one match equally.
     * @param matchValue Integer ranking the closeness of the match.
     * @param matchedTag Instance of the child tag that made the match.
     */
    protected void setMatch(int matchValue, StyleTag matchedTag) {
        if (matchValue < 1) {
            return;
        }
        if (_matchedTag == null || matchValue > _matchValue) {
            _matchValue = matchValue;
            _matchedTag = matchedTag;
        }
    }

    /**
     *  Sets the scope at which this tag's resources will be made available.
     *  By default session scope is used.
     *
     *  @param scope One of application, session, request, or page.
     */
    public void setScope (String scope)
        throws JspException
    {
        if (scope == null)
            throw new JspTagException("style:styling tag invalid scope attribute of null");
        else if (scope.equals("application"))
            _scope = PageContext.APPLICATION_SCOPE;
        else if (scope.equals("session"))
            _scope = PageContext.SESSION_SCOPE;
        else if (scope.equals("request"))
            _scope = PageContext.REQUEST_SCOPE;
        else if (scope.equals("page"))
            _scope = PageContext.PAGE_SCOPE;
        else
            throw new JspTagException("style:styling tag invalid scope attribute=" + scope);
    }

    /**
     * Set the type attribute - defaults to "affiliate".
     * @param type One of affiliate, abtest
     */
    public void setType (String type)
        throws JspException
    {
        if (type == null || type.toLowerCase().equals("affiliate")) {
            _type = "affiliate";
            _styleAttribute = STYLE_ATTRIBUTE_AFFILIATE;
            _alternateStyleParam = this.STYLE_PARAM_AFFILIATE;
        }
        else if (type.toLowerCase().equals("abtest"))  {
            _type = "abtest";
            _styleAttribute = STYLE_ATTRIBUTE_ABTEST;
            _alternateStyleParam = this.STYLE_PARAM_ABTEST;
        }
        else {
            throw new JspTagException("style:styling tag invalid type attribute=" + type);
        }
    }

    /**
     * @return Current style name
     */
    public String getStyle ()
    {
        return _style;
    }

    /**
     * If a style is supplied in the request parameters, use that one and store it in the given
     * scope.  If not, check in all scopes for an existing style.  If those also don't exist,
     * stick with the default value.
     */
    private void determineStyle ()
    {
        final PageContext pageContext = (PageContext) getJspContext();
        final HttpServletRequest req = (HttpServletRequest) pageContext.getRequest();

        // get style from a request parameter
        final String styleParam = req.getParameter(STYLE_PARAM);
        if (styleParam != null) {
            _style = styleParam;
            getJspContext().setAttribute(_styleAttribute, styleParam, _scope);
            return;
        }

        // get style from an alternate request param based on type
        final String alternateStyleParam = req.getParameter(_alternateStyleParam);
        if (alternateStyleParam != null) {
            _style = alternateStyleParam;
            getJspContext().setAttribute(_styleAttribute, alternateStyleParam, _scope);
            return;
        }

        // get previously set style attribute
        final String styleAttribute = (String) getJspContext().findAttribute(_styleAttribute);
        if (styleAttribute != null) {
            _style = styleAttribute;
            return;
        }

        // get the default style
        final String defaultStyle = getDefaultStyle(pageContext, req);
        if (defaultStyle != null) {
            _style = defaultStyle;
            getJspContext().setAttribute(_styleAttribute, defaultStyle, _scope);
            return;
        }
    }

    /**
     * Return the default style for this request, based either on affiliate or ab test,
     * depending on attributes and whether either is supported by the context.
     */
    private String getDefaultStyle (PageContext pageContext, HttpServletRequest req)
    {
        // get the default style by affiliate or by tag (if supported, otherwise exit)
        final AffiliateManager affiliateManager;
        try {
            final ContextManager contextManager = ContextManager.getContextManager(pageContext);
            if (contextManager == null) {
                return null;
            }
            affiliateManager = contextManager.getAffiliateManager();
            if (affiliateManager == null) {
                return null;
            }
        }
        catch (JspException e) {
            return null;
        }

        // determine default style by affiliate
        if (_type.equals("affiliate")) {
            final String affiliateStyle = affiliateManager.getAffiliateStyle(req);
            if (affiliateStyle != null) {
                return affiliateStyle;
            }

        }
        return null;
    }

    /** Value of the current highest-ranking match */
    private int _matchValue;

    /** Child tag that made the current highest-ranking match */
    private StyleTag _matchedTag;

    /** Scope at which style will be made available. */
    private int _scope = PageContext.SESSION_SCOPE;

    /** Style to use; passed in via request param and stored by scoped attribute */
    private String _style = "default";

    /** Attribute for determining where the default style will come from - affiliate or abtest */
    private String _type = "affiliate";

    /** Key of the style attribute in the given scope if type == affiliate */
    private static final String STYLE_ATTRIBUTE_AFFILIATE = StylingTag.class.getName() + "_style_affiliate";

    /** Key of the style attribute in the given scope if type == abtest */
    private static final String STYLE_ATTRIBUTE_ABTEST = StylingTag.class.getName() + "_style_abtest";

    /** Which style attribute to use; changed by setting type */
    private String _styleAttribute = STYLE_ATTRIBUTE_AFFILIATE;

    /** Request parameter name */
    private final String STYLE_PARAM = "style";

    /** Request parameter name for type == affiliate only (overriden by STYLE_PARAM) */
    private final String STYLE_PARAM_AFFILIATE = "style_affiliate";

    /** Request parameter name for type == abtest only (overriden by STYLE_PARAM) */
    private final String STYLE_PARAM_ABTEST = "style_abtest";

    /** If STYLE_PARAM is not present, look for this alternate param, changed by setting type */
    private String _alternateStyleParam = STYLE_PARAM_AFFILIATE;
}