package com.threerings.jsp.taglib.choose;

import java.io.IOException;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.tagext.JspFragment;
import javax.servlet.jsp.tagext.SimpleTagSupport;

/**
 * Tag representing an if statement based upon visitor type, to be included within a
 * {@link VisitorChooseTag}.  If the current visitor type matches the type attribute,
 * the body of this tag will be evaluated.  If two or more {@link VisitorCaseTag}s in the same
 * choose tag match, the first will be evaluated.  If the type attribute is set to "default", it
 * will only be evaluated if no other case tag matches.
 */
public class VisitorCaseTag extends SimpleTagSupport
{
    /**
     * Invoke the body of this tag.  This will be called by the parent upon selecting this
     * tag as a match.
     */
    public void doBody()
        throws JspException, IOException
    {
        final JspFragment body = getJspBody();
        body.invoke(null);
    }

    /**
     * Assess how well the type attribute matches the visitor type in the parent tag, then
     * register this value with the parent {@link VisitorChooseTag}.
     */
    @Override
    public void doTag()
        throws JspException, IOException
    {
        // this must appear within the correct parent tag
        if (getParent() == null || !(getParent() instanceof VisitorChooseTag)) {
            throw new JspTagException("Visitor case tags only valid within a visitor choose tag.");
        }
        parentTag = (VisitorChooseTag) getParent();

        // if this matches the current style (or the default), notify parent
        int matchValue = matchType(parentTag.getVisitorType());
        if (matchValue > 0) {
            // notify the parent we've matched
            parentTag.setMatch(matchValue, this);
        }
    }

    /**
     * Set the type to match against, or "default" to weakly match any type
     */
    public void setType (String type)
    {
        _type = type;
    }

    /**
     * Returns a value if a match was found between the type attribute and the current visitor
     * type, or zero for no match.  The higher the value, the better the match.
     *
     * @param currentType Current visitor type to match type attribute against
     * @return 0 if no match is found, 1 for a default tag match, 2 for a matching type.
     */
    protected int matchType (String currentType)
        throws JspException
    {
        if (_type == null) {
            return 0;
        }

        // if the type attribute is default, return a weak match value
        if (_type.equalsIgnoreCase("default")) {
            return 1;
        }

        if (currentType == null) {
            return 0;
        }

        // if the type attribute matches, return a strong match value
        if (currentType.equalsIgnoreCase(_type)) {
            return 2;
        }
        return 0;
    }

    /** type attribute to match against the current visitor type */
    private String _type;

    /** The parent tag */
    private VisitorChooseTag parentTag;
}