package com.threerings.jsp.taglib.form;

import java.io.IOException;
import java.io.StringWriter;
import java.io.Writer;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.DynamicAttributes;
import javax.servlet.jsp.tagext.JspFragment;
import javax.servlet.jsp.tagext.SimpleTagSupport;

import com.threerings.jsp.taglib.xhtml.AttributeMap;

/**
 * Displays an HTML form element.  Upon submit, allow child validation tags to register any
 * errors.  If any are found, re-display the form body including validation errors instead of
 * forwarding to the location given by the action attribute.
 *
 * TODO add tag for performing content switches on _validationError for forms with a blank action.
 * TODO support relative actions or actions in other contexts by using response.sendRedirect()
 *      instead of RequestDispatcher.forward()?
 * TODO offer protection from multiple submits or abuse of the back button?
 */
public class FormTag extends SimpleTagSupport
    implements DynamicAttributes
{
    /**
     * Invoke the body to trigger child validation tags.  If any one of them reports an error,
     * do not forward to the action url and instead display the form body.
     */
    @Override
    public void doTag ()
        throws JspTagException, IOException, JspException
    {
        // determine if we are submitting or just viewing
        final PageContext pageContext = (PageContext) getJspContext();
        final HttpServletRequest request = (HttpServletRequest) pageContext.getRequest();
        final HttpServletResponse response = (HttpServletResponse) pageContext.getResponse();
        setSubmitting(request);

        // if there are no client side validators, have the form submit to "action" as normal.
        if (!_useValidators) {
            _attributes.put("action", _action);
        }

        // verify the body is not null
        final JspFragment body = getJspBody();
        if (body == null) {
            final Writer out = getJspContext().getOut();
            out.write("<form " + _attributes.toString() + " />\n");
            return;
        }

        // send the body output to a string buffer so we can trigger it without outputting yet.
        // validators will fire at this point and register validationError if there is one.
        final StringWriter bodyWriter = new StringWriter();
        body.invoke(bodyWriter);
        final StringBuffer bodyBuffer = bodyWriter.getBuffer();

        // if there were no errors on submit, forward on to the action location
        if (isSubmitting() && !_validationError) {

            // do not forward if action is not supplied
            if (_action == null || _action.length() == 0) {
                if (_showAgain) {
                    displayForm(bodyBuffer);
                }
                return;
            }

            if (_forward) {
                final ServletContext servletContext = pageContext.getServletContext();
                final RequestDispatcher dispatcher = servletContext.getRequestDispatcher(_action);
                if (dispatcher == null) {
                    throw new JspTagException("FormTag could not create dispatcher to " + _action);
                }
                try {
                    // reset the form to 'not submitted'
                    request.setAttribute(SUBMIT_ACTION_PARAM, "false");
                    dispatcher.forward(request, response);
                } catch (ServletException e) {
                    throw new JspTagException("FormTag could not forward request to " + _action
                        + ":\n" + e);
                }
            }
        }
        else {
            displayForm(bodyBuffer);
        }
    }

    /**
     * Set the URL that the form will submit to.
     */
    public void setAction (String action)
    {
        // don't add to the rest of the params until we know there are no client side validators.
        _action = action;
    }

    /**
     * Display the form again when the form has been submitted (only applies when action==null)
     */
    public void setShowAgain (String showAgain)
    {
        _showAgain = Boolean.valueOf(showAgain);
    }

    /**
     * Sets whether or not the tag should take it upon itself to try to forward the request.
     * This defaults to true.
     */
    public void setForward (String forwardAttribute)
    {
        // check for null, since we want to default to true
        if (forwardAttribute != null) {
            _forward = Boolean.valueOf(forwardAttribute);
        }
    }

    /**
     * Sets whether or not the tag has client-side validators. If it does, submit to the same page
     * first and validate before forwarding to the action url. Defaults to false.
     */
    public void setUseValidators (String useValidators)
    {
        _useValidators = Boolean.valueOf(useValidators);
    }

    /**
     * Add a dynamic attribute to our list
     */
    public void setDynamicAttribute (String namespace, String name, Object value)
    {
        if (value != null) {
            _attributes.put(name, value.toString());
        }
    }

    /**
     * Called by child validator tags if there is an error during form validation.  Does
     * not accept a value - sets validation error to true.
     */
    protected void setValidationError ()
    {
        _validationError = true;
    }

    /**
     * Print the HTML for the form element plus body.
     * @param bodyBuffer Contains the HTML for the tag body.
     */
    private void displayForm (StringBuffer bodyBuffer)
        throws JspException, IOException
    {
        // open the form tag (no action attribute)
        final Writer out = getJspContext().getOut();
        out.write("<form " + _attributes.toString() + ">\n");

        // display a hidden submit action input
        out.write("<input type=\"hidden\" name=\"" + SUBMIT_ACTION_PARAM + "\" value=\"true\" />\n");

        // display the body output and close the form
        out.write(bodyBuffer.toString());
        out.write("</form>\n");
    }

    /**
     * Returns true if the form is being submitted.  Called by child validator tags to check
     * whether they should fire or not.
     */
    protected boolean isSubmitting ()
    {
        return isSubmitting;
    }

    /**
     * Look for our secret hidden submitting param to check whether the form is being submitted
     * or simply viewed.  Action param may be overridden by an action attribute.
     */
    private void setSubmitting (HttpServletRequest request)
    {
        String action = (String) request.getAttribute(SUBMIT_ACTION_PARAM);
        if (action == null) {
            action = request.getParameter(SUBMIT_ACTION_PARAM);
        }
        if (action != null && action.equals("true")) {
            isSubmitting = true;
        }
        else {
            isSubmitting = false;
        }
    }

    /** Was there an error during form validation? */
    private boolean _validationError = false;

    /** Is the form being submitted, or is this the first time page load? */
    private boolean isSubmitting;

    /** Location to submit the form to - don't let it get lost in the dynamic attributes. */
    private String _action;

    /** Collection of dynamic attributes */
    private final AttributeMap _attributes = new AttributeMap();

    /** Display the form again when the form has been submitted (only applies when action==null) */
    private boolean _showAgain = false;

    private boolean _forward = true;

    private boolean _useValidators = false;

    /** This will be stored in a hidden input to tell us when form is being submitted */
    private static final String SUBMIT_ACTION_PARAM = "formSubmitted";
}
