package com.threerings.jsp.taglib.form;

import java.io.IOException;
import java.io.Writer;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.JspFragment;
import javax.servlet.jsp.tagext.SimpleTagSupport;

/**
 * Abstract class for validating a specific form field.  Extending classes must appear inside
 * a {@link FormTag} and implement the isValid() method.
 * TODO extend to add client-side validation via AJAX?
 */
public abstract class ValidatorTag extends SimpleTagSupport
{
    /**
     * If the form is being submitted, check for vaildation errors and display the body
     * of this tag if there is one.  Also notify the form tag so it can cancel submitting.
     */
    @Override
    public void doTag ()
        throws JspTagException, IOException, JspException
    {
        // must appear within a Form tag
        final FormTag formTag = (FormTag) findAncestorWithClass(this, FormTag.class);
        if (formTag == null) {
            throw new JspTagException("Validator tags must appear within a Form tag.");
        }

        // do nothing unless the form is being submitted
        if (!formTag.isSubmitting()) {
            return;
        }

        // select the value of the form field from the request parameters
        final PageContext pageContext = (PageContext) getJspContext();
        final HttpServletRequest request = (HttpServletRequest) pageContext.getRequest();
        final String fieldValue = request.getParameter(field);

        // if value is invalid, tell the form about it and output the body
        if (!isValid(fieldValue)) {
            formTag.setValidationError();
            final Writer out = getJspContext().getOut();
            final JspFragment body = getJspBody();
            body.invoke(out);
        }
    }

    /**
     * Set the name of the form field to validate.
     */
    public void setField (String field)
    {
        this.field = field;
    }

    /**
     * Defines logic to determine if a particular value is of legal format.
     * @param fieldValue Value of the submitted request parameter specified by the field attribute.
     * @return True if the fieldValue is valid, otherwise false
     */
    protected abstract boolean isValid (String fieldValue);

    /** Name of the field to validate */
    protected String field;
}
