package com.threerings.jsp.taglib.i18n.currency;

import java.io.IOException;
import java.util.Locale;

import javax.servlet.http.HttpSession;
import javax.servlet.jsp.JspContext;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.SimpleTagSupport;

import com.threerings.jsp.taglib.i18n.DefaultLocaleFilter;

/**
 * Provide a tag to support currency quotations for localization.  The tag will take a currency amount, and check
 * whether it is in a currency that is suitable for the current locale.  If it is, then the contents of the tag will be ignored,
 * otherwise the contents of the tag will be output.  The QuoteTag can be used to insert a value in an appropriate
 * currency within the body of this tag.
 */
public class CurrencyQuoteTag extends SimpleTagSupport
{
    private LocalizableMoney amount;

    /**
     * Add a currency quote service to the session.  This must occur before the tag is processed.
     */
    public static void addQuoteServiceToSession (HttpSession session, QuoteService service) {
        session.setAttribute(QuoteService.class.getCanonicalName(), service);
    }

    /**
     * Get the currency quote service from the session.
     */
    private QuoteService quoteService () throws JspException {
        final QuoteService service = (QuoteService) getJspContext().findAttribute(QuoteService.class.getCanonicalName());
        if (service == null) throw new JspException("no QuoteService found in context");
        return service;
    }

    /**
     * Add a currency map to the session.  This must occur before the tag is processed.
     * @param session
     * @param map
     */
    public static void addCurrencyMapToSession(HttpSession session, LocaleCurrencyMap map) {
        session.setAttribute(LocaleCurrencyMap.class.getCanonicalName(), map);
    }

    /**
     * Get a currency map from the session.
     */
    private LocaleCurrencyMap currencyMap () throws JspException {
        final LocaleCurrencyMap map =
            (LocaleCurrencyMap) getJspContext().findAttribute(LocaleCurrencyMap.class.getCanonicalName());
        if (map == null) throw new JspException("no LocaleCurrencyMap found in context");
        return map;
    }

    /**
     * Set the amount for the quote.  This is expected to be done through the setting of an attribute.
     * @throws JspException
     */
    public void setAmount(String name) throws JspException {
        // the name is the name of a context variable containing the amount
        final Object amount = getJspContext().findAttribute(name);
        if (amount == null) throw new JspException("no variable in context called "+name);
        if (amount instanceof LocalizableMoney) this.amount = (LocalizableMoney) amount;
        else this.amount = parseMoney(amount.toString());
    }

    /**
     * Returns true if the money object supplied is in a currency that is foreign to the display locale.
     * @throws JspException
     */
   private boolean needsQuote (LocalizableMoney money) throws JspException {
        return currencyMap().isCurrencyForeign(displayLocale(), money.currency());
    }

    /**
     * Get the locale in which output is to be displayed.
     * @throws JspException
     */
    private Locale displayLocale() throws JspException {
        final Locale found = (Locale) getJspContext().findAttribute(DefaultLocaleFilter.LOCALE_ATTRIBUTE);
        if (found == null) throw new JspException("locale not found in context - is an appropriate filter configured?");
        return found;
    }

    /**
     * Parse a string containing a monetary value and return a LocalizableMoney object.
     * @throws JspException
     */
    private LocalizableMoney parseMoney(String toParse) throws JspException {
        return quoteService().parseMoney(toParse);
    }

    /**
     * Actually generate the currency quote.
     */
    private LocalizableMoney quote (LocalizableMoney money) throws JspException {
        return quoteService().quote(currencyMap().currencyForLocale(displayLocale()), money);
    }

    /**
     * Process an instance of this tag in a jsp page.
     */
    @Override
    public void doTag () throws JspException, IOException {
        if (needsQuote(amount)) {
            final JspContext context = getJspContext();
            QuoteTag.setQuote(context, quote(amount));
            getJspBody().invoke(context.getOut());
        }
    }
}
