/*
 * Copyright 2007 Three Rings Design, Inc.
 * Copyright 1999,2004 The Apache Software Foundation.
 *
 * This file's implementation is partially based upon code from
 * Jakarta Taglibs's BundleTag, authored by Tim Dawson <tdawson@wamnet.com>
 */

package com.threerings.jsp.taglib.i18n.resources;

import com.threerings.i18n.I18n;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.PageContext;

/**
 * Implement an (empty) tag that loads a resource bundle for a specific
 * locale. The locale may either be specified as explicit tag attributes,
 * or as a {@link java.util.Locale} instance placed in either the page,
 * session, and application scope(s).
 */
public class BundleTag extends GenericResourceTag
{
	/**
     *  Locates and prepares an I18n instance for use within this request by
     *  i18n tags.  The resource bundle locale is determined by:
     *   - Finding a bundle that matches the explicit locale, if provided
     *   - Finding a bundle matching the provided Locale attribute {@link #setLocaleRef(String)}
     */
    private I18n findBundle()
        throws JspException
    {
        /* The bundle name is required. */
        if (_baseName == null)
            throw new JspTagException("i18n:bundle tag, a baseName attribute must be specified.");

        return new I18n(_baseName, getLocale());
    }

    @Override
    public void doTag ()
        throws JspException
    {
        /* The bundle name is required. */
        if (_baseName == null)
            throw new JspTagException("i18n:bundle tag: A basename attribute must be specified");

        /* Allocate an i18n instance for the user's locale. */
        _i18n = findBundle();

        /* tell i18n what type of localization to use for images */
        _i18n.setImgLocalize(_imgLocalize);

        /* Cache our i18n bundle in the requested scope. */
        ResourceHelper.setI18n((PageContext) getJspContext(), _i18n, getScope());
    }

    /**
     * Get the I18n instance created by this bundle tag, used by nested tags.
     *
     * @return I18n instance.
     */
    public I18n getI18n () {
    	return _i18n;
    }

    /**
     *  Sets the baseName of the bundle that the I18n will use when
     *  retrieving keys for this page.
     *
     *  @param baseName {@link I18n} base name.
     */
    public final void setBaseName(String baseName)
    {
        _baseName = baseName;
    }

    /**
     * Sets the default type of localization for images in this bundle.
     */
    public void setImgLocalize (String imgLocalize) {
        _imgLocalize = imgLocalize;
    }

    /** Cached I18n instance. */
    private I18n _i18n;

    /** Resource base name. */
    private String _baseName = null;

    /** Default localization method for images in this bundle */
    private String _imgLocalize = "none";
}
