/*
 * Copyright 2007 Three Rings Design, Inc.
 */

package com.threerings.jsp.taglib.i18n.resources;

import com.threerings.i18n.I18n;

/**
 * Interface a method of translating a filename into a different locale, for use by the.  Several
 * default implementations are provided.
 */
public interface FilenameMapper
{
    /**
     * Convert a file path based on the locale from the given i18n resource.
     * @param src The source path for the file (eg /images/image.gif)
     * @param i18n Internationalization bundle with current local
     * @return Translated image src
     */
    public abstract String translateSrc (String src, I18n i18n);

    /**
     * Localization is not performed; src is returned unchanged.
     */
    public static class NoMapper implements FilenameMapper
    {
        public String translateSrc (String src, I18n i18n)
        {
            return src;
        }
    }

    /**
     * Automatically translate the file path by adding a locale extension before the file type
     * extension.  Eg: /images/image.gif -> /images/image.en.gif
     */
    public static class ExtensionMapper implements FilenameMapper
    {
        public String translateSrc (String src, I18n i18n)
        {
            try {

                int lastDot = src.lastIndexOf(".");
                int lastSlash = src.lastIndexOf("/");

                /* path including trailing slash */
                final String path = src.substring(0, lastSlash + 1);
                /* filename sans extension */
                final String filename = src.substring(lastSlash + 1, lastDot);
                /* extension including dot */
                final String extension = src.substring(lastDot);

                /* automatically add localization extension to the filename */
                final String locale = i18n.getLocale().toString();
                return path + filename + "." + locale + extension;
            }
            catch (IndexOutOfBoundsException e) {
                return null;
            }
        }
    }

    /**
     * Like ExtensionLocalizer, but adds the locale with an underscore to the end of the
     * filename.  Eg: /images/image.gif -> /images/image_en.gif
     */
    public static class UnderscoreMapper implements FilenameMapper
    {
        public String translateSrc (String src, I18n i18n)
        {
            try {

                int lastDot = src.lastIndexOf(".");
                int lastSlash = src.lastIndexOf("/");

                /* path including trailing slash */
                final String path = src.substring(0, lastSlash + 1);
                /* filename sans extension */
                final String filename = src.substring(lastSlash + 1, lastDot);
                /* extension including dot */
                final String extension = src.substring(lastDot);

                /* automatically add localization extension to the filename */
                final String locale = i18n.getLocale().toString();
                return path + filename + "_" + locale + extension;
            }
            catch (IndexOutOfBoundsException e) {
                return null;
            }
        }
    }

    /**
     * Automatically translates the file path by adding a locale directory before the file
     * name.  Eg: /images/image.gif -> /images/en/image.gif
     */
    public class PathMapper implements FilenameMapper
    {
        public String translateSrc (String src, I18n i18n)
        {
            try {
                final String locale = i18n.getLocale().toString();
                final int doubleSlash = src.lastIndexOf("://");
                final int lastSlash = src.lastIndexOf("/");

                // if the last slash is containined in http://, there is no filename or trailing slash
                if (doubleSlash > 0 && lastSlash == doubleSlash + 2) {
                    return src + "/" + locale + "/";
                }

                // path before the filename, including a trailing slash
                final String path = src.substring(0, lastSlash + 1);
                // filename with extension
                final String filename = src.substring(lastSlash + 1);

                return path + locale + "/" + filename;
            }
            catch (IndexOutOfBoundsException e) {
                return null;
            }
        }
    }


    /**
     * Automatically translates the file path by adding a ?lang=LOCALE param to the end
     * of it.
     */
    public class ParamMapper implements FilenameMapper
    {
        public String translateSrc (String src, I18n i18n)
        {
            try {
                final String locale = i18n.getLocale().toString();

                // if a query string already exsits, append to it with an &
                int lastQuestionMark = src.lastIndexOf("?");
                if (lastQuestionMark > -1) {
                    return src + "&amp;lang=" + locale;
                }
                else {
                    return src + "?lang=" + locale;
                }

            }
            catch (IndexOutOfBoundsException e) {
                return null;
            }
        }
    }

}
