/*
 * Copyright 2007 Three Rings Design, Inc.
 */

package com.threerings.jsp.taglib.i18n.resources;

import java.io.IOException;
import java.io.Writer;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.SimpleTagSupport;

import com.threerings.i18n.I18n;

/**
 * Translate a given url based on the locale and localization method.
 * TODO: allow web application to implement custom FilenameMappers.
 */
public class FilenameTag extends SimpleTagSupport
{
    @Override
    public void doTag ()
        throws JspException, IOException
    {
        // find the resource bundle
        final I18n i18n = ResourceHelper.getI18n((PageContext)getJspContext());
        if (i18n == null) {
            throw new JspTagException("i18n:filename tag, no bundle available for use.");
        }
        final String localizedFile;

        // skip localization for this locale
        if (_skipLocale != null && _skipLocale.equals(i18n.getLocale().toString())) {
            localizedFile = _src;
        }
        // perform localization using the correct mapper
        else {
            FilenameMapper filenameMapper = getFilenameMapper(_localize);
            localizedFile = filenameMapper.translateSrc(_src, i18n);
        }

        final Writer output = getJspContext().getOut();
        output.write(localizedFile);
    }

    /**
     * Set the method of image localization to use.  Some legal values are none, path, extension.
     */
    public void setLocalize (String localize)
    {
        if (localize == null) {
            _localize = null;
        }
        else {
            _localize = localize.toLowerCase();
        }
    }

    /**
     * Set the name of the file we will translate.  May include a full path.
     */
    public void setSrc (String src)
    {
        _src = src;
    }

    /**
     * Set the locale in which to skip localization.  Optional, null by default.
     */
    public void setSkipLocale (String skipLocale)
    {
        _skipLocale = skipLocale;
    }

    /**
     * Return an instance of the correct localization method class based on the localize string.
     */
    protected FilenameMapper getFilenameMapper (String localize)
        throws JspException
    {
        // fetch the resource mapper implementation based on the localize attribute
        if (_localize.equals("none")) {
            return new FilenameMapper.NoMapper();
        }
        else if (_localize.equals("path")) {
            return new FilenameMapper.PathMapper();
        }
        else if (_localize.equals("extension")) {
            return new FilenameMapper.ExtensionMapper();
        }
        else if (_localize.equals("underscore")) {
            return new FilenameMapper.UnderscoreMapper();
        }
        else if (_localize.equals("param")) {
            return new FilenameMapper.ParamMapper();
        }
        else {
            throw new JspException("Unknown localize method: " + _localize);
        }
    }

    /** Determines method of localization for the src attribute. */
    protected String _localize = null;

    /** Filename to translate */
    protected String _src;

    /** If set to a locale, do not translate while in that locale */
    protected String _skipLocale = null;
}
