// $Id $
//

package com.threerings.jsp.taglib.i18n.resources;

import java.util.Locale;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.SimpleTagSupport;

import com.threerings.jsp.taglib.i18n.DefaultLocaleFilter;
import com.threerings.jsp.taglib.i18n.LocaleUtils;

/**
 * Generic implementation of a localizable resource-vending tag.
 *
 * Supports setting the scope of the resource, and a reference
 * to the user's locale (or a specific locale override).
 *
 * Used to vend resources to tags within the same scope -- ie,
 * a bundle tag vends an i18n bundle that will be used by i18n text tags.
 */
public abstract class GenericResourceTag extends SimpleTagSupport {
    /**
     *  Sets the scope at which this tag's resources will be made available.
     *  By default page scope is used.
     *
     *  @param scope One of application, session, request, or page.
     */
    public void setScope (String scope) throws JspException
    {
        if (scope == null)
            throw new JspTagException("i18n:bundle tag invalid scope attribute of null");
        else if (scope.equals("application"))
            _scope = PageContext.APPLICATION_SCOPE;
        else if (scope.equals("session"))
            _scope = PageContext.SESSION_SCOPE;
        else if (scope.equals("request"))
            _scope = PageContext.REQUEST_SCOPE;
        else if (scope.equals("page"))
            _scope = PageContext.PAGE_SCOPE;
        else
            throw new JspTagException("i18n:bundle tag invalid scope attribute=" + scope);
    }


    /**
     * Returns the resource's intended scope, as defined in {@link PageContext}
     */
    protected int getScope () {
        return _scope;
    }


    /**
     * Provides an attribute key which will be used to find
     * the preferred user Locale within the page context.
     * Defaults to {@link DefaultLocaleFilter#LOCALE_ATTRIBUTE}.
     *
     * @param localeRef Locale attribute key.
     */
    public void setLocaleRef (String localeRef) {
        _localeAttribute = localeRef;
    }


    /**
     *  Sets an explicit, overriding locale that should be used when retrieving
     *  localized resources.
     *
     *  @param localeString A locale string, in standard 'lang_COUNTRY@variant' format.
     */
    public void setLocale (String localeString)
        throws JspException
    {
        final Locale locale = LocaleUtils.parseLocale(localeString);
        if (locale == null)
            throw new JspTagException("i18n:bundle, unparsable locale: " + localeString);

        _localeOverride = locale;
    }

    /**
     * Get the expected locale -- either supplied explicitly by {@link #setLocale(String)},
     * or located via the attribute name provided by {@link #setLocaleRef(String)}.
     *
     * @return Locale to use for localization.
     * @throws JspException
     */
    protected Locale getLocale () throws JspException {
        /* If there is an explicitly defined locale, try to use it. */
        if (_localeOverride != null)
            return _localeOverride;

        /* Search the context for the locale attribute. */
        final Locale locale = (Locale) getJspContext().findAttribute(_localeAttribute);

        /* A locale is required. */
        if (locale == null)
            throw new JspTagException("i18n:bundle tag, no request locale could be determined.");
        else
            return locale;
    }

    /** Scope at which this bundle will be available to i18n message tags. */
    private int _scope = PageContext.PAGE_SCOPE;

    /** Locale instance attribute name. */
    private String _localeAttribute = DefaultLocaleFilter.LOCALE_ATTRIBUTE;

    /** Manually set locale. */
    private Locale _localeOverride = null;
}
