/*
 * Copyright 2007 Three Rings Design, Inc.
 */
package com.threerings.jsp.taglib.i18n.resources;

import java.io.IOException;
import java.io.Writer;
import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.DynamicAttributes;

import com.threerings.i18n.I18n;

/**
 * This class implements an image tag that allows you to use an i18n bundle to internationalize
 * content in a web page.  The BundleTag must first be used in order to ensure that the
 * proper bundle is loaded.  Extends the {@link FilenameTag} to gain src and localize attributes,
 * and the getFilenameMapper method, but overrides doTag() to print an <img> tag.
 */
public class ImageTag extends FilenameTag
    implements DynamicAttributes
{
    @Override
    public void doTag ()
        throws JspException, IOException
    {
        // find the resource bundle
        final I18n i18n = ResourceHelper.getI18n((PageContext)getJspContext());
        if (i18n == null) {
            throw new JspTagException("i18n:image tag, no bundle available for use.");
        }

        // find the localization method set in the i18n bundle if not set in the tag
        if (_localize == null){
            setLocalize(i18n.getImgLocalize());
        }

        // output the image tag
        outputImageTag(getFilenameMapper(_localize), i18n);
    }

    /**
     * Build the image tag and write to output.
     * @param filenameMapper Method of translating image src and mouseover path
     * @param i18n Localization bundle for fetching translations.
     */
    private void outputImageTag (FilenameMapper filenameMapper, I18n i18n)
        throws JspException, IOException
    {
        // src is required; translate it according to our localize method
        final String src = filenameMapper.translateSrc(_src, i18n);
        attributes.put("src", src);

        // translate alt and title attributes if present
        if (_localizeText) {
            final String alt = attributes.get("alt");
            if (alt != null) {
                attributes.put("alt", i18n.gettext(alt));
            }
            final String title = attributes.get("title");
            if (title != null) {
                attributes.put("title", i18n.gettext(title));
            }
        }

        // translate second onmouseover src and add javascript events
        if (_mouseover != null) {
            _mouseover = filenameMapper.translateSrc(_mouseover, i18n);
            // change to the alternate src on mouseover, and back to the original on mouseout
            attributes.put("onmouseover", "this.src = '" + _mouseover + "';");
            attributes.put("onmouseout", "this.src = '" + src + "';");
        }

        // perform the ie 6 workaround for transparent pngs if required
        if (_isTransparentPng) {
            iePngWorkaround();
        }

        // collate our attributes and generate the img tag
        String attributeString = "";
        for (String attribute : attributes.keySet()) {
            attributeString += attribute + "=\"" + attributes.get(attribute) + "\" ";
        }
        final String outputTag = "<img " + attributeString + " />";

        // write the tag to the output
        final Writer output = getJspContext().getOut();
        output.write(outputTag);
    }

    /**
     * IE 6 has support for png alpha transparency in the form of a DX filter.  If
     * the browser is IE6 or earlier, apply the workaround.  In this case, width
     * and height attributes are required.
     */
    private void iePngWorkaround ()
        throws JspException
    {
        // width and height are required
        final String width = attributes.get("width");
        final String height = attributes.get("height");
        if (width == null || height == null) {
            throw new JspException("Transparent PNG Img tag must supply width and height.");
        }

        // style will be overridden, so error if it is set
        final String style = attributes.get("style");
        if (style != null) {
            throw new JspException("Transparent PNG Img tag may not use a style attribute.");
        }

        final HttpServletRequest req = (HttpServletRequest)((PageContext)getJspContext()).getRequest();
        final String agent = req.getHeader("User-Agent");
        if (agent != null) {
            if (agent.contains("Win")) {
                final Matcher matcher = ie6Pattern.matcher(agent);
                if (matcher.find()) {
                    final String major = matcher.group(1);
                    final String minor = matcher.group(2);
                    if (major.equals("6") && minor.equals("0")) {
                        applyIePngWorkaround("ie6");
                    }
                    else if (major.equals("5") && (minor.equals("0") || minor.equals("5"))) {
                        applyIePngWorkaround("ie5");
                    }
                }
            }
        }
    }

    /**
     * Change the image's src to a blank image and apply the original src as a
     * DXImageTransform filter.  Also look for mouseover and mouseout values and apply
     * a similar fix using javascript when performing mouseover src changes.
     */
    private void applyIePngWorkaround (String browser)
        throws JspException
    {
        final String width = attributes.get("width");
        final String height = attributes.get("height");
        final String src = attributes.get("src");
        final String filter = buildIePngFilter(browser, width, height, src);

        // set width and height again in style for good measure
        final String style =
            "width: " + width + "px; " +
            "height: " + height + "px; " +
            "filter: " + filter + ";";

        attributes.put("style", style);
        attributes.put("src", _transparentSrc);

        // apply the workaround on mouseover / mouseout
        if (_mouseover != null) {

            final String mouseoverFilter = buildIePngFilter(browser, width, height, _mouseover);
            attributes.put("onmouseover", "this.style.filter='" + mouseoverFilter + "';");
            attributes.put("onmouseout", "this.style.filter='" + filter + "';");
        }
    }

    /**
     * Construct and return the definition string for the ie png workaround filter, which
     * differs by browser.
     */
    private String buildIePngFilter (String browser, String width, String height, String src)
    {
        String filter = "progid:DXImageTransform.Microsoft.AlphaImageLoader(" +
            "src=" + src;

        // ie6 uses set width and height, can't use scale because it results in warping
        // ie 5.x can't use set width and height, so we'll scale the image instead
        if (browser.equals("ie6")) {
            filter += " width=" + width + " " + "height=" + height + ")";
        }
        else {
            filter += "), sizingMethod=scale";
        }
        return filter;
    }

    /**
     * Image location path for displaying during onmouseover events.  Javascript onmouseover and
     * onmouseout attributes will be generated if this is non-null.  Translated in the same manner
     * as src.
     */
    public void setMouseover (String mouseover)
    {
        _mouseover = mouseover;
    }

    /**
     * Will the title and alt attributes be translated?
     */
    public void setLocalizeText (String localizeText)
    {
        if (localizeText == null || localizeText.toLowerCase().equals("true")) {
            _localizeText = true;
        }
        else {
            _localizeText = false;
        }
    }

    /**
     * Override the default file to use for the transparency src for the PNG transparency fix.
     */
    public void setTransparentSrc (String transparentSrc)
    {
        _transparentSrc = transparentSrc;
    }

    /**
     * Set whether the IE6 png tansparent fix is applied (defaults to false)
     */
    public void setIsTransparentPng (String value)
    {
        if (value == null || value.equals("false")) {
            _isTransparentPng = false;
        }
        else {
            _isTransparentPng = true;
        }
    }

    /**
     * Additional attributes such as width & style are not translated.
     */
    public void setDynamicAttribute (String uri, String localName, Object value)
    {
        attributes.put(localName, value.toString());
    }

    /** Pattern for locating msie 6 or lower */
    private static final Pattern ie6Pattern = Pattern.compile("MSIE\\s+(\\d+)\\.(\\d+)\\s*");

    /** Map of dynamic attributes */
    private Map<String,String> attributes = new HashMap<String,String>();

    /** Untranslated src to use for mouseover image swapping events */
    private String _mouseover;

    /** should we localize alt and title text? */
    private boolean _localizeText = true;

    /** Whether or not to apply the IE6 transparent png fix */
    private boolean _isTransparentPng = false;

    /** Location of the transparent image for png transparency fix */
    private String _transparentSrc = "/images/blank.gif";
}