package com.threerings.jsp.taglib.i18n.resources;

import java.io.IOException;
import java.util.Locale;

import javax.servlet.http.HttpSession;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.JspFragment;
import javax.servlet.jsp.tagext.SimpleTagSupport;

import com.threerings.i18n.I18n;
import com.threerings.jsp.taglib.i18n.DefaultLocaleFilter;
import com.threerings.jsp.taglib.i18n.LocaleUtils;

/**
 * Tag representing an if statement based upon a particular locale, to be included within a
 * {@link ChooseLocaleTag}. If the current locale in the i18n bundle matches the locale attribute,
 * the body of this tag will be evaluated. If two or more {@link LocaleTag}s in the same choose
 * tag match, the first will be evaluated. If the locale attribute is set to "default", it will
 * only be evaluated if no other locale tag matches.
 *
 *
 */
public class LocaleTag extends SimpleTagSupport
{
    /**
     * Invoke the body of this tag.  This will be called by the parent upon selecting this
     * tag as a match.
     */
    public void doBody()
        throws JspException, IOException
    {
        final JspFragment body = getJspBody();
        if (body != null) {
            body.invoke(null);
        }
    }

    /**
     * Assess how well the locale attribute matches the locale in the i18n tag, then register
     * this value with the parent {@link ChooseLocaleTag}.
     */
    @Override
    public void doTag()
        throws JspException, IOException
    {
        // this must appear within a choose locale tag
        if (getParent() != null && !(getParent() instanceof ChooseLocaleTag)) {
            throw new JspTagException("Locale tags only valid within a choose locale tag.");
        }
        parentChooseLocaleTag = (ChooseLocaleTag) getParent();

        // if we are in the right locale (or the default locale), notify parent
        int matchValue = matchLocale();
        if (matchValue > 0) {
            // notify the parent we've matched
            parentChooseLocaleTag.setMatch(matchValue, this);
        }
    }

    /**
     * Set the locale to match against, or "default" to weakly match any locale.
     */
    public void setLocale (String locale)
        throws JspException
    {
        if (locale == null) {
            _locales = new String[0];
        }
        _locales = locale.split(",");
    }

    /**
     * Which part of the locale to match. Can be one of internal (the default - meaning the local
     * we have assigned which is usually only language, eg "en") or country.
     */
    public void setPart (String part)
        throws JspException
    {
        Parts newPart = Parts.valueOf(part);
        if (newPart == null) {
            throw new JspException("Unknonwn part: " + part);
        } else {
            _part = newPart;
        }
    }

    /**
     * Returns a value if a match was found between the locale attribute and the visitor's locale,
     * or zero for no match.  The higher the value, the better the match.
     *
     * @param locale Current locale from an i18n resource bundle
     * @return 0 if no match is found, 1 for a default tag match, 2 for a matching locale.
     */
    protected int matchLocale ()
        throws JspException
    {
        // if the locale attribute is default, return a weak match value
        for (String locale : _locales) {
            if (locale.equalsIgnoreCase("default")) {
                return 1;
            }
        }

        // match against the country from the original locale string
        if (_part == Parts.country) {
            HttpSession session = ((PageContext)getJspContext()).getSession();
            Locale originalLocale = (Locale)session.getAttribute(DefaultLocaleFilter.LOCALE_ATTRIBUTE);
            if (originalLocale == null) {
                return 0;
            }
            String country = LocaleUtils.parseLocale(originalLocale.toString()).getCountry();
            for (String locale : _locales) {
                if (locale.equalsIgnoreCase(country)) {
                    return 2;
                }
            }

        // match against the locale we assigned which must be one that we support
        } else {
            final I18n i18n = ResourceHelper.getI18n((PageContext)getJspContext());
            if (i18n == null) {
                throw new JspTagException("i18n:locale tag, no bundle available for use.");
            }
            Locale internalLocale = i18n.getLocale();
            if (internalLocale == null) {
                return 0;
            }
            for (String locale : _locales) {
                if (locale.equalsIgnoreCase(internalLocale.toString())) {
                    return 2;
                }
            }
        }

        // no match
        return 0;
    }

    /** locale attribute, split by commas */
    private String[] _locales;

    /** internal (default) or country */
    private Parts _part = Parts.internal;

    /** The parent choose tag */
    protected ChooseLocaleTag parentChooseLocaleTag;

    /**
     * Legal values for the part parameter.
     */
    protected static enum Parts {
        internal, country;
    }
}
