//
// $Id: $

package com.threerings.jsp.taglib.i18n.resources;

import java.io.IOException;
import java.io.StringWriter;
import java.io.Writer;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.JspFragment;
import javax.servlet.jsp.tagext.JspTag;
import javax.servlet.jsp.tagext.SimpleTagSupport;

import com.threerings.i18n.I18n;

/**
 * A body tag to handle composition of localizable messages,
 * in both plural and singular form.
 *
 * The BundleTag must first be used in order to ensure that a translation
 * bundle is loaded.
 */
public class MessageTag extends SimpleTagSupport
{
    /**
     * A simple child tag to provide format string arguments to the message tag.
     */
    public static class ArgumentTag extends SimpleTagSupport {
        @Override
        public void doTag ()
            throws JspException, IOException
        {
            /* Fetch our tag body */
            final JspFragment body = getJspBody();
            final StringWriter writer = new StringWriter();
            body.invoke(writer);

            /* Get our parent MessageTag */
            final JspTag parentTag;
            final MessageTag messageTag;
            parentTag = getParent();
            if (!(getParent() instanceof MessageTag))
                throw new JspTagException("i18n:arg tags must be declared within an i18n:message tag.");

            /* Add the argument text to the message */
            messageTag = (MessageTag) parentTag;
            messageTag.addArgument(writer.toString());

            /* If this is the count argument, convert the body text to a long
             * and inform our enclosing Message tag */
            if (_isCount) {
            	try {
                	long n = Long.parseLong(writer.toString());
                	messageTag.setCount(n);
            	} catch (NumberFormatException e) {
            		throw new JspTagException("i18n:arg could not parse count attribute value: " + writer.toString());
            	}
            }
        }

        /**
         * Marks this tag instance as the 'count' argument, used to decide
         * between plural and singular forms.
         *
         * @param isCount True if this argument is to be used for the count argument.
         */
        public void setCount (boolean isCount) {
        	_isCount = isCount;
        }

        /** Is this tag the 'count' argument? */
        private boolean _isCount;
    }

    @Override
    public void doTag ()
        throws JspException, IOException
    {
    	String translation;

        /* Find our translation bundle */
        final I18n i18n = findI18n();

        /* Invoke our tag body */
        final JspFragment body = getJspBody();
        final StringWriter writer = new StringWriter();
        body.invoke(writer);

        /* Is the text missing? */
        if (_text == null)
            throw new JspTagException("i18n:message missing an i18n:text subtag.");

        /* Translate the message */
        if (_plural == null)
        	translation = i18n.gettext(_text, _arguments.toArray());
        else
        	translation = i18n.ngettext(_text, _plural, _count, _arguments.toArray());

        /* Output the text */
        final Writer output = getJspContext().getOut();
        output.write(translation);
    }

    /**
     * Set the localizable message key. If plural is
     * true, sets the plural representation of the translation
     * message.
     *
     * Called by {@link TextTag}
     *
     * @param text The localizable string message key.
     * @param plural Set to true if this is a plural form.
     */
    protected void setText (String text, boolean plural)
    {
    	if (plural)
    		_plural = text;
    	else
    		_text = text;
    }

    /**
     * Set the count, used to choose between plural and singular forms.
     *
     * Called by {@link ArgumentTag}
     *
     * @param count Item count.
     */
    protected void setCount (long count) {
    	_count = count;
    }

    /**
     * Append an argument to the format string argument list. The arguments
     * will be passed to {#I18n#gettext}.
     *
     * Called by @{link TextTag}
     *
     * @param argument Additional format string argument
     */
    protected void addArgument (String argument) {
        _arguments.add(argument);
    }

    /**
     *  @return The I18n bundle to use.
     */
    private I18n findI18n()
        throws JspException
    {
        I18n i18n = ResourceHelper.getI18n((PageContext)getJspContext());
        if (i18n == null)
            throw new JspTagException("i18n:message tag, no bundle available for use.");

        return i18n;
    }

    /** The message key. */
    private String _text;

    /** The message's plural message key, or null if no plural form exists. */
    private String _plural;

    /** The plural count -- used to choose between plural and singular forms */
    private long _count;

    /** Message arguments, if any */
    private List<String> _arguments = new ArrayList<String>();
}
