/*
 * Copyright 2007 Three Rings Design, Inc.
 * Copyright 1999,2004 The Apache Software Foundation.
 *
 * This file's implementation is partially based upon code from
 * Jakarta Taglibs's MessageTag, authored by Tim Dawson <tdawson@wamnet.com>
 */

package com.threerings.jsp.taglib.i18n.resources;

import java.io.IOException;
import java.io.StringWriter;
import java.io.Writer;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.JspFragment;
import javax.servlet.jsp.tagext.JspTag;
import javax.servlet.jsp.tagext.SimpleTagSupport;

import com.threerings.i18n.I18n;

/**
 * This class implements a body tag that allows you to use an i18n bundle
 * to internationalize content in a web page.
 *
 * The BundleTag must first be used in order to ensure that the proper bundle
 * is loaded.
 */
public class TextTag extends SimpleTagSupport
{

	/**
	 * Normalize a translation key.
	 * This method cleans up text as follows:
	 *     - Remove any trailing/starting whitespace and newlines.
	 *     - Strip all '[\r\n]+ *' matches, replacing with a single whitespace.
	 *         Foo\n    Bar -> Foo Bar
	 * @param text
	 * @return
	 */
    private String normalizeKey (String text) {
    	Matcher match = IGNORE_WHITESPACE.matcher(text.trim());
    	return match.replaceAll(" ");
    }

    @Override
    public void doTag ()
        throws JspException, IOException
    {
        /* Fetch our tag body */
    	final JspFragment body = getJspBody();
    	final StringWriter writer = new StringWriter();
        body.invoke(writer);
        String key = normalizeKey(writer.toString());

        /*
         * If we have a MessageTag parent, provide it with the localization key
         * and let it handle message output.
         *
         * Otherwise, translate and output the text directly.
         */
        final JspTag parent = getParent();
        if (parent != null && parent instanceof MessageTag) {
            /* Send the message to our parent tag for translation. */
            MessageTag messageTag = (MessageTag)parent;

            /* Is this plural form? */
            messageTag.setText(key, _plural);
        } else {
            /* Translate the message directly */
            translate(key);
        }
    }

    /**
     * Set the "plurality" of this tag. If set to true, the tag is assumed to
     * represent the plural form of a translation message. This is only useful
     * when the text is composed by an enclosing {@link MessageTag}.
     *
     * @param plural Set tag "plurality"
     */
    public void setPlural (boolean plural) {
    	_plural = plural;
    }

    /**
     * Translate and write the message to the JSP output.
     *
     * @param key Message key.
     */
    private void translate(String key)
    	throws JspException, IOException
    {
        /* Find the resource bundle. */
        final I18n i18n = ResourceHelper.getI18n((PageContext)getJspContext());
        if (i18n == null)
        	throw new JspTagException("i18n:text tag, no bundle available for use.");

        /* Translate the message. */
        final String translation = i18n.gettext(key);
        final Writer output = getJspContext().getOut();
        output.write(translation);
    }

    /** True if the tag represents a plural localization key */
    private boolean _plural = false;

    /** Matches ignorable white space in a translation key */
    private static final Pattern IGNORE_WHITESPACE = Pattern.compile("[ \t]*[\r\n]+[\r\n\t ]*");
}
