/*
 * Copyright 2007 Three Rings Design, Inc.
 */
package com.threerings.jsp.taglib.tracking;

import java.io.IOException;
import java.util.UUID;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.codec.digest.DigestUtils;

import com.threerings.jsp.taglib.affiliates.ABTestFilter;
import com.threerings.jsp.taglib.utils.CookieUtils;

/**
 * Filter for handing out tracking ids to visitors.  Random UUIDs are generated with {@link UUID}.
 * The presence of a tracking id may be used to determine previous visitors as in {@link ABTestFilter}.
 *
 * The class supports the following init parameters:
 *     strip-cookie-domain: If true, the first hostname element (eg www.) will be removed from the
 *     cookie domain.
 */
public class DefaultTrackingFilter implements Filter {

    /** If this request param is set to "true" the user will be treated like a new one */
    protected static final String NEW_USER_OVERRIDE_PARAM = "isNewUser";

    /** Request attribute name. This attribute is set to the request's tracking id. */
    protected static final String TRACKING_ATTRIBUTE = DefaultTrackingFilter.class.getName() + ".TrackingId";

    /** First Session attribute name.  Set to true if a new tracking id was assigned during this session.
     * {@link ABTestFilter} uses this to determine returning visitors vs new ones. */
    protected static final String FIRST_SESSION_ATTRIBUTE = DefaultTrackingFilter.class.getName() + ".FirstSession";

    /** User's tracking id cookie name. */
    protected static final String TRACKING_COOKIE = DigestUtils.md5Hex(DefaultTrackingFilter.class.getName() + ".Cookie");

    /**
     * Returns true if we can't find a tracking cookie or if it's their first session in the cookie
     */
    public static boolean isNewVisitor (HttpServletRequest req)
    {
        Visitor visitor = DefaultTrackingFilter.getVisitor(req);
        return visitor == null || visitor.isNewVisitor();
    }

    /**
     * Retrieve a Visitor containing the tracking id, or null if a tracking id has not been assigned.
     */
    public static Visitor getVisitor (HttpServletRequest req)
    {
        final String trackingId = (String) req.getSession().getAttribute(TRACKING_ATTRIBUTE);
        if (trackingId == null) {
            return null;
        }
        final String newVisitorAttribute = (String) req.getSession().getAttribute(FIRST_SESSION_ATTRIBUTE);
        final boolean newVisitor;
        if (newVisitorAttribute != null && newVisitorAttribute.equals("true")) {
            newVisitor = true;
        }
        else {
            newVisitor = false;
        }
        final Visitor visitor = new Visitor(trackingId, newVisitor);
        return visitor;
    }

    /**
     * Filter initialization.
     */
    public void init(FilterConfig config)
        throws ServletException
    {
        String param;

        // Should we strip the hostname off our domain? Default to true.
        param = config.getInitParameter("strip-cookie-domain");
        if (param == null || param.equalsIgnoreCase("true")) {
            _cookieStripDomain = true;
        } else {
            _cookieStripDomain = false;
        }
    }

    /**
     * Filter destruction.
     */
    public void destroy() {
        // Nothing
    }

    /**
     * Retrieve or set a tracking id.
     */
    public void doFilter(ServletRequest servletRequest, ServletResponse servletResponse, FilterChain chain)
        throws IOException, ServletException
    {
        final HttpServletRequest req = (HttpServletRequest) servletRequest;
        final HttpServletResponse resp = (HttpServletResponse) servletResponse;

        // Assign or retrieve the tracking id
        setTrackingId(req, resp);

        // Pass the request on
        chain.doFilter(req, resp);
    }

    /**
     * If there is already a tracking id for this session, do nothing.  If there is a tracking
     * id cookie from a previous session, retrieve it.  If there is no tracking id, assign one.
     */
    protected void setTrackingId (HttpServletRequest req, HttpServletResponse resp)
    {
        // for testing we are pretending to be a new user or a returning user
        if ("true".equals(req.getParameter(NEW_USER_OVERRIDE_PARAM))) {
            assignNewTrackingId(req, resp);
            return;
        } else if ("false".equals(req.getParameter(NEW_USER_OVERRIDE_PARAM))) {
            assignNewTrackingId(req, resp);
            req.getSession().setAttribute(FIRST_SESSION_ATTRIBUTE, "false");
            return;
        }

        // If there is a tracking id already in the session, do nothing
        final String sessionTrackingId = (String) req.getSession().getAttribute(TRACKING_ATTRIBUTE);
        if (sessionTrackingId != null) {
            return;
        }

        // If there is a tracking id in the cookies, put it in the session
        final String cookieTrackingId = CookieUtils.getCookie(req, TRACKING_COOKIE);
        if (cookieTrackingId != null) {
            req.getSession().setAttribute(TRACKING_ATTRIBUTE, cookieTrackingId);
            return;
        }

        // No previous tracking id found; assign a new one and store it in cookies and session
        assignNewTrackingId(req, resp);
    }

    /**
     * Create a new tracking id and store it in the session and cookies.
     */
    protected void assignNewTrackingId (HttpServletRequest req, HttpServletResponse resp)
    {
        final UUID trackingId = UUID.randomUUID();
        req.getSession().setAttribute(TRACKING_ATTRIBUTE, trackingId.toString());
        CookieUtils.setCookie(req, resp, TRACKING_COOKIE, trackingId.toString(), 2*365, _cookieStripDomain);
        req.getSession().setAttribute(FIRST_SESSION_ATTRIBUTE, "true");
    }

    /** Should we remove the server hostname (eg www) from the cookie? */
    private Boolean _cookieStripDomain;
}