/*
 * Copyright 2007 Three Rings Design, Inc.
 */
package com.threerings.jsp.taglib.utils;

import org.apache.commons.configuration.Configuration;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.PageContext;

import com.threerings.jsp.taglib.affiliates.AffiliateManager;

/**
 * Abstract container for other manager objects.  Can fetch or add itself to a ServletContext,
 * and contains overridable methods to return ConfigurationManager and AffiliateManager instances.
 */
public abstract class ContextManager
{

    /**
     * Fetch an instance of this class from the servlet context
     */
    public static ContextManager getContextManager (ServletContext servletContext)
        throws ServletException
    {
        ContextManager manager = (ContextManager) servletContext.getAttribute(CONTEXT_KEY);
        if (manager == null) {
            throw new ServletException(
                "ContextManager couldn't find manager instance in the servlet context.");
        }
        return manager;
    }

    /**
     * Convenience function for fetching from within a tag.  Accepts a PageContext and throws
     * JspException if not called from within a page, or if the context manager is not found.
     */
    public static ContextManager getContextManager (PageContext pageContext)
        throws JspException
    {
        final ServletContext servletContext = pageContext.getServletContext();
        ContextManager manager = (ContextManager) servletContext.getAttribute(CONTEXT_KEY);
        if (manager == null) {
            throw new JspException(
                "ContextManager couldn't find manager instance in the page servlet context.");
        }
        return manager;
    }

    /**
     * Add this object to the given context
     */
    public void addToContext (ServletContext context)
    {
        context.setAttribute(CONTEXT_KEY, this);
    }

    /**
     * Remove any instance of this class from the servlet context
     */
    public static void removeFromContext (ServletContext context)
    {
        context.removeAttribute(CONTEXT_KEY);
    }

    /**
     * Return the Configuration, or null if one is not available
     */
    public Configuration getConfiguration ()
    {
        return null;
    }

    /**
     * Return the AffiliateManager, or null if one is not available
     */
    public AffiliateManager getAffiliateManager ()
    {
        return null;
    }

    /** This class will be stored in the context under this key */
    private static final String CONTEXT_KEY = ContextManager.class.getCanonicalName();
}