//
// $Id: ABTest.java 265 2013-05-08 22:42:36Z mdb $

package com.threerings.util;

import java.io.InputStream;
import java.io.InputStreamReader;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.logging.Level;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.JsonArray;
import com.google.gson.JsonDeserializationContext;
import com.google.gson.JsonDeserializer;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParseException;
import com.google.gson.reflect.TypeToken;

import static com.threerings.jsp.taglib.Log.log;

/**
 * Data on the conditions for applying and the various modes of an ab test.
 */
public class ABTest
{
    /**
     * The value from {@link #affiliates} that indicates to allow unaffiliated direct traffic
     * through.
     */
    public static final String DIRECT_AFFILIATE = "(direct)";

    /**
     * Reads ABTests out of a json file at the given path on the classpath.
     *
     * @return a Map of test id to ABTest, or an empty map if the path couldn't be located or the
     * json couldn't be parsed.
     */
    public static Map<String, ABTest> loadTests(String path)
    {
        InputStream in = ABTest.class.getClassLoader().getResourceAsStream(path);
        if (in == null) {
            log.warning("Unable to locate abtests on classpath [path=" + path + "]");
            return new HashMap<String, ABTest>();
        }
        Type abtestsType = new TypeToken<Map<String, ABTest>>(){}.getType();
        Type groupsType = new TypeToken<Map<String, Group>>(){}.getType();
        Gson gson = new GsonBuilder().
             registerTypeAdapter(abtestsType, new ABTestMapDeserializer()).
             registerTypeAdapter(Date.class, new DateArrayDeserializer()).
             registerTypeAdapter(groupsType, new GroupSetDeserializer()).create();
        try {
            return gson.fromJson(new InputStreamReader(in,  "UTF-8"), abtestsType);
        } catch (Exception e) {
            log.log(Level.WARNING, "Failed parsing abtest json [path=" + path + "]", e);
            return new HashMap<String, ABTest>();
        }
    }

    public static class Group implements Cloneable {
        public Group(String id, String description) {
            this.id = id;
            this.description = description;
        }

        public Set<String> components = new HashSet<String>();

        public String id;

        public String description;

        @Override
        public String toString () {
            return id;
        }
    }

    /** The id of the test. */
    public String id;

    /** The affiliates to test or null if applied to only direct. */
    public Set<String> affiliates;

    /** The locales to test or null if applied to all locales. */
    public Set<String> locales;

    /**
     * The paths in the site to apply the test to. If empty, this test didn't apply to the website.
     */
    public Set<String> paths = Collections.emptySet();

    /**
     * Maps from group id to the group itself. A group id defaults to
     * <code>&lt;test id&gt;&lt;group name&gt;</code> or is just group name if
     * <code>prefixGroupIdsWithTestId</code> is set to false.
     */
    public Map<String, Group> groups;

    /** True if the test should be applied to allVisitors, false if only to new. */
    public boolean allVisitors;

    /** When the test started or null if it's open-ended. */
    public Date start;

    /** When the test ends or null if it's open-ended. */
    public Date end;

    /**
     * If the group ids should be prefixed with the id of the test, or if they should remain
     * unmolested.
     */
    public boolean prefixGroupIdsWithTestId = true;

    public boolean isActive()
    {
        Date now = new Date();
        return (start == null || start.before(now)) && (end == null || end.after(now));
    }

    /**
     * Creates a Map from the ABTest name to the ABTest. Needs a custom deserializer to set the
     * id on the ABTest object.
     */
    protected static class ABTestMapDeserializer implements JsonDeserializer<Map<String, ABTest>> {
        public Map<String, ABTest> deserialize (JsonElement json, Type type,
            JsonDeserializationContext ctx) throws JsonParseException {
            Map<String, ABTest> result = new HashMap<String, ABTest>();
            for (Map.Entry<String, JsonElement> entry : json.getAsJsonObject().entrySet()) {
                ABTest test = ctx.deserialize(entry.getValue(), ABTest.class);
                test.id = entry.getKey();
                if (test.groups == null || test.groups.size() <= 1) {
                    throw new IllegalArgumentException(
                        test.id + ".groups must contain at least 2 values");
                }

                if (test.prefixGroupIdsWithTestId) {
                    Map<String, Group> prefixedGroups = new HashMap<String, Group>();
                    for (Map.Entry<String, Group> group : test.groups.entrySet()) {
                        // Build the group id out of the test id and the group name
                        String prefixed = test.id + group.getKey();
                        group.getValue().id = prefixed;
                        prefixedGroups.put(prefixed, group.getValue());
                    }
                    test.groups = prefixedGroups;
                }
                result.put(entry.getKey(), test);
            }

            return result;
        }
    }

    protected static class GroupSetDeserializer implements JsonDeserializer<Map<String, Group>> {
        public Map<String, Group> deserialize (JsonElement json, Type type,
            JsonDeserializationContext ctx) throws JsonParseException {
            List<JsonObject> elems;
            // If groups was an array in the json, that means we have multiple pieces of test that
            // we want to run at the same time. Multiply them out to make distinct groups for each
            // of the combinations.
            if (json.isJsonArray()) {
                JsonArray asArray = json.getAsJsonArray();
                elems = new ArrayList<JsonObject>(json.getAsJsonArray().size());
                for (JsonElement jsonElement : asArray) {
                    elems.add(jsonElement.getAsJsonObject());
                }
            } else {
                elems = new ArrayList<JsonObject>();
                elems.add(json.getAsJsonObject());
            }
            Set<Group> current = new HashSet<Group>();
            for (JsonObject components : elems) {
                if (current.isEmpty()) {
                    for (Map.Entry<String, JsonElement> entry : components.entrySet()) {
                        Group g = new Group(entry.getKey(), entry.getValue().getAsString());
                        g.components.add(entry.getKey());
                        current.add(g);
                    }
                } else {
                    Set<Group> multipliedResult = new HashSet<Group>();
                    for (Map.Entry<String, JsonElement> entry : components.entrySet()) {
                        String newId = entry.getKey();
                        String newDescription = entry.getValue().getAsString();
                        for (Group group : current) {
                            Group combined = new Group(group.id + newId, group.description + " "
                                + newDescription);
                            combined.components.add(entry.getKey());
                            combined.components.addAll(group.components);
                            multipliedResult.add(combined);
                        }
                    }
                    current = multipliedResult;
                }
            }
            Map<String, Group> result = new HashMap<String, Group>(current.size());
            for (Group group : current) {
                result.put(group.id, group);
            }
            return result;
        }
    }

    /**
     * Deserializes a Date from an array rather than the default deserializer that uses String.
     */
    protected static class DateArrayDeserializer implements JsonDeserializer<Date>{
        public Date deserialize (JsonElement json, Type typeOfT, JsonDeserializationContext ctx)
            throws JsonParseException {
            JsonArray array = json.getAsJsonArray();
            _cal.clear();
            _cal.set(array.get(0).getAsInt(), array.get(1).getAsInt() - 1, array.get(2).getAsInt());
            return _cal.getTime();
        }

        protected final Calendar _cal = Calendar.getInstance();
    }
}
