package com.threerings.jsp.taglib.xhtml;

import java.io.IOException;
import java.io.StringWriter;
import java.io.UnsupportedEncodingException;
import java.io.Writer;
import java.net.URLEncoder;
import java.util.HashMap;
import java.util.Map;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.tagext.DynamicAttributes;
import javax.servlet.jsp.tagext.JspFragment;
import javax.servlet.jsp.tagext.SimpleTagSupport;

/**
 * Prints a flash object tag. Dynamic attributes become param tags.
 */
public class FlashTag extends SimpleTagSupport
    implements DynamicAttributes
{
    @Override
    public void doTag ()
        throws JspException, IOException
    {
        // invoke the body on a dummy writer to trigger child tag flashvar registration
        final JspFragment body = getJspBody();
        if (body != null) {
            body.invoke(new StringWriter());
        }

        // print the output
        final Writer out = getJspContext().getOut();
        out.write("\n" + getOpeningObjectTag());
        out.write(getParams());
        out.write(getOpeningEmbedTag());
        out.write("\t</embed>\n");
        out.write("</object>\n");
    }

    /**
     * Dynamic attributes will become both object param tags and embed attributes.
     */
    public void setDynamicAttribute (String namespace, String name, Object value)
    {
        if (value != null) {
            _dynamicParams.put(name, value.toString().trim());
        }
    }

    public void setSrc (String src) {
        _src = src;
    }

    public void setId (String id) {
        _id = id;
    }

    /** Width attribute is optional */
    public void setWidth (String width) {
        _width = width;
    }

    /** Height attribute is optional */
    public void setHeight (String height) {
        _height = height;
    }

    public void setAllowScriptAccess (String allowScriptAccess) {
        _allowScriptAccess = allowScriptAccess;
    }

    public void setClassid (String classid) {
        _classid = classid;
    }

    public void setCodebase (String codebase) {
        _codebase = codebase;
    }

    public void setFlashvars (String flashvars) {
        _flashvars = flashvars;
    }

    /**
     * Called by child FlashvarTags to add flashvar variables.
     */
    public void addFlashvar (String name, String value)
    {
        _flashvarParams.put(name, value);
    }

    /**
     * Generate and return html for the object tag.
     */
    private String getOpeningObjectTag ()
    {
        StringBuffer tag = new StringBuffer();
        Map<String,String> attributes = new HashMap<String,String>();

        if (_id != null) {
            attributes.put("id", _id);
        }
        if (_width != null) {
            attributes.put("width", _width);
        }
        if (_height != null) {
            attributes.put("height", _height);
        }
        attributes.put("allowScriptAccess", _allowScriptAccess);
        attributes.put("classid", _classid);
        attributes.put("codebase", _codebase);

        // build opening object tag
        tag.append("\n<object ");
        for (String attribute : attributes.keySet()) {
            tag.append(attribute + "=\"" + attributes.get(attribute) + "\" ");
        }
        tag.append(">\n");
        return tag.toString();
    }

    /**
     * Generate and return html for the param elements of the object tag.
     */
    private String getParams ()
        throws JspException
    {
        StringBuffer paramString = new StringBuffer();

        // calculate flashvars and add it to dynamic params
        try {
            if (_flashvars == null) {
                StringBuffer flashvarsString = new StringBuffer();
                for (String key : _flashvarParams.keySet()) {
                    String value = _flashvarParams.get(key);
                    if (value != null && value.trim().length() > 0) {
                        value = URLEncoder.encode(value, "utf-8");
                        flashvarsString.append(key + "=" + value + "&amp;");
                    }
                }
                if (flashvarsString.length() > 1) {
                    _flashvars = flashvarsString.substring(0, flashvarsString.length() - 5);
                } else {
                    _flashvars = "";
                }
            }
        } catch (UnsupportedEncodingException e) {
            throw new JspException("Could not encode flashvars in utf-8.");
        }
        _dynamicParams.put("flashvars", _flashvars);

        paramString.append("\t<param name=\"movie\" value=\"" + _src + "\"/>\n");

        // add dynamic params
        for (String param : _dynamicParams.keySet()) {
            if (_dynamicParams.get(param).trim().length() > 0) {
                paramString.append("\t<param name=\"" + param + "\" value=\""
                    + _dynamicParams.get(param) + "\"/>\n");
            }
        }
        return paramString.toString();
    }

    /**
     * Generate and return html for the embed tag.
     */
    private String getOpeningEmbedTag ()
    {
        StringBuffer tag = new StringBuffer();

        Map<String, String> attributes = new HashMap<String, String>();

        attributes.put("src", _src);
        if (_width != null) {
            attributes.put("width", _width);
        }
        if (_height != null) {
            attributes.put("height", _height);
        }
        attributes.put("pluginspace", _pluginspace);
        attributes.put("ntype", _ntype);

        // build opening object tag
        tag.append("\t<embed ");
        for (String attribute : attributes.keySet()) {
            tag.append(attribute + "=\"" + attributes.get(attribute) + "\" ");
        }

        // add dynamic params
        for (String param : _dynamicParams.keySet()) {
            if (_dynamicParams.get(param).trim().length() > 0) {
                tag.append(param + "=\"" + _dynamicParams.get(param) + "\" ");
            }
        }

        tag.append(">\n");
        return tag.toString();
    }

    /**
     * flashvar variables will be sent in on this tag
     */
    public static class FlashvarTag extends SimpleTagSupport
    {
        @Override
        public void doTag()
            throws JspException, IOException
        {
            // this must appear within the correct parent tag
            _parentTag = (FlashTag) findAncestorWithClass(this, FlashTag.class);
            if (_parentTag == null) {
                throw new JspTagException("Flashvar tags only valid within a flash tag.");
            }

            // tell the parent about this variable
            _parentTag.addFlashvar(_name, _value);
        }

        public void setName (String name) {
            _name = name;
        }
        public void setValue (String value) {
            _value = value;
        }

        private String _name;
        private String _value;
        private FlashTag _parentTag;
    }

    /** dynamic attributes become dynamic param tags */
    private final Map<String, String> _dynamicParams = new HashMap<String,String>();

    /** flashvar variables from child FlashvarTags */
    private final Map<String, String> _flashvarParams = new HashMap<String, String>();

    /** Optional id */
    private String _id;

    /** Required src / movie param */
    private String _src;

    /** Optional width */
    private String _width;

    /** Optional height */
    private String _height;

    /** Optional will override child flashvar tags if present */
    private String _flashvars;

    /** Optional allowScriptAccess defaults to "always" */
    private String _allowScriptAccess = "always";

    /** Direct browsers to the flash plugin */
    private String _classid = "clsid:d27cdb6e-ae6d-11cf-96b8-444553540000";

    /** Browsers will automatically download the plugin from here if necessary */
    private String _codebase = "http://active.macromedia.com/flash7/cabs/swflash.cab#version=10,0,0,0";

    /** embed plugin url */
    private String _pluginspace = "http://www.macromedia.com/go/getflashplayer";

    /** embed type */
    private String _ntype = "application/x-shockwave-flash";
}
